/*
 *  DaysOffSchool.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 */


using System;
using System.Collections;
using System.IO;
using System.Xml;


namespace gnutu {
	
	public struct DayOffSchool {
		public Date   date;
		public string description;
	}
	
	
	
	/*
	 *  DaysOffSchool class
	 */
	
	public class DaysOffSchool {
		static public ArrayList data;
		
		
		/* Opens and loads days off school database */
		static public bool Open ()
		{
			StreamReader reader;
			data = new ArrayList ();
			
			try {
				reader = new StreamReader (FilesPaths.DaysOffSchoolFile (), System.Text.Encoding.UTF8);
			} catch {
				return false;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			DayOffSchool day   = new DayOffSchool ();
			
			try {
				while (read.Read ()) {
					if ((read.Name == "day") && (read.NodeType == XmlNodeType.Element)) {
						day = new DayOffSchool ();
						
						day.date        = new Date (read.GetAttribute ("date"), false);
						day.description = read.ReadString ();
						
						data.Add (day);
					}
				}
			} catch {
				data = new ArrayList ();
				
				reader.Close ();
				return false;
			}
			
			
			reader.Close ();
			return true;
		}
		
		
		/* Closes and saves days off school database */
		static public void Close ()
		{
			XmlTextWriter f = new XmlTextWriter (FilesPaths.DaysOffSchoolFile (), System.Text.Encoding.UTF8);
			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "DaysOffSchool", "http://www.gnutu.org");
			
			int i = 0;
			foreach (DayOffSchool day in data) {
				f.WriteStartElement ("day");
				f.WriteAttributeString ("date", day.date.ToString (false));
					f.WriteString (day.description);
				f.WriteEndElement ();
				
				i++;
			}
			
			f.WriteEndElement ();
			f.WriteEndDocument ();
			f.Close ();
		}
		
		
		/* Removes all the data */
		static public void Clear ()
		{
			data.Clear ();
		}
		
		
		/* Add an entry (day off school [day]) */
		static public void Add (DayOffSchool day)
		{
			data.Add (day);
		}
		
		
		/* Is selected day [date] off school */
		static public bool IsDayOffSchool (Date date)
		{
			foreach (DayOffSchool day in data) {
				if ((day.date.day == date.day) && (day.date.month == date.month) && (day.date.year == date.year))
					return true;
			}
			
			return false;
		}
		
		
		/* Returns selected [date] day off school) */
		static public string GetDayDescription (Date date)
		{
			foreach (DayOffSchool day in data) {
				if ((day.date.day == date.day) && (day.date.month == date.month) && (day.date.year == date.year))
					return day.description;
			}
			
			return "";
		}


		/* Sets day [date] as day off school + description [description] */
		static public void SetDayOffSchool (Date date, string description)
		{
			DayOffSchool NewDay = new DayOffSchool ();
			NewDay.date = date;
			NewDay.description = description;

			int i = 0;
			
			foreach (DayOffSchool day in data) {
				if ((day.date.day == date.day) && (day.date.month == date.month) && (day.date.year == date.year)) {
					data[i] = NewDay;
					return;
				}

				i++;
			}

			Add (NewDay);
		}


		/* Removes day [date] off school */
		static public void RemoveDayOffSchool (Date date)
		{
			foreach (DayOffSchool day in data) {
				if ((day.date.day == date.day) && (day.date.month == date.month) && (day.date.year == date.year)) {
					data.Remove (day);
					return;
				}
			}
		}
		
		
		/* Returns days off school from selected month [month] */
		static public ArrayList GetMonth (int month)
		{
			month++;
			ArrayList rtn = new ArrayList ();
			
			foreach (DayOffSchool day in data)
				if (day.date.month == month)
					rtn.Add (day);
			
			return rtn;
		}
	}
}
