/*
 *  LessonsSchedule.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 */


using System;
using System.Collections;
using System.IO;
using System.Xml;
using Mono.Unix;


namespace gnutu {
    
	public struct Skin {
	    public string name;
	    public string path;
	}
	
    
	
	public class LessonsScheduleData {
		static public ArrayList [] data;

		
		/* Opens and loads lessons schedule */
		static public bool Open ()
		{
			data = new ArrayList [] {new ArrayList (), new ArrayList (), new ArrayList (), 
						new ArrayList (), new ArrayList (), new ArrayList (),
						new ArrayList (), new ArrayList ()};
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.LessonsScheduleFile (), System.Text.Encoding.UTF8);
			} catch  {
				return false;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			ArrayList     day  = new ArrayList ();
			
			int i = 0;
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						if ((read.Name == "day") && (read.NodeType == XmlNodeType.Element)) {
							day = new ArrayList ();
						}
						
						if ((read.Name == "subject") && (read.NodeType == XmlNodeType.Element)) {
							day.Add (read.ReadString ());
						}
						
						if ((read.Name == "day") && (read.NodeType == XmlNodeType.EndElement)) {
							data [i] = day;
							i++;
						}
					}
				}
			} catch {
				reader.Close ();
				return false;
			}
			
			
			i = 1;
			while (i < 7) {
				data [i] = Fill (data [i], data [0].Count);
				i++;
			}
				
		
			reader.Close ();
			return true;
		}
		
		
		/* Closes and saves lessons schedule */
		static public void Close ()
		{
			XmlTextWriter f = new XmlTextWriter (FilesPaths.LessonsScheduleFile (), System.Text.Encoding.UTF8);
			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "LessonsSchedule", "http://www.gnutu.org");
			
			int i = 0;
			foreach (ArrayList arr in data) {
				f.WriteStartElement ("day");
					foreach (string day in arr) {
						f.WriteStartElement ("subject");
							f.WriteString (day);
						f.WriteEndElement ();
					}
				f.WriteEndElement ();
				
				i++;
			}
			
			f.WriteEndElement ();
			f.WriteEndDocument ();
			f.Close ();
		}
		
		
		/* Return selected day [day] */
		static public ArrayList Get (int day)
		{
			return data [day];
		}
		
		
		/* Insert data [list] to selected day [day] */
		static public void Set (int day, ArrayList list)
		{
			data [day] = list;
		}
		
		
		/* Returns lessons list */
		static public ArrayList GetLessonsList ()
		{
			ArrayList list = new ArrayList ();
			
			int i = 1;
			while (i < 7) {
				foreach (string subj in data [i])
					list = AddSubject (list, subj);
				
				i++;
			}
			
			return list;
		}
		
		
		static public ArrayList AddSubject (ArrayList list, string subject)
		{
			if (subject.Length == 0)
				return list;
			
			foreach (string subj in list) {
				if (subj == subject)
					return list;
			}
		
			list.Add (subject);
				
			return list;
		}
		
		
		static private ArrayList Fill (ArrayList list, int count)
		{
			int i = count - list.Count;
			
			while (i > 0) {
				list.Add ("");
				
				i--;
			}
			
			return list;
		}
		
		
		static private string ColumnText (int id)
		{
			switch (id) {
				case 0: return "";
				case 1: return Catalog.GetString ("Monday");  
				case 2: return Catalog.GetString ("Tuesday"); 
				case 3: return Catalog.GetString ("Wednesday"); 
				case 4: return Catalog.GetString ("Thursday");
				case 5: return Catalog.GetString ("Friday");   
				case 6: return Catalog.GetString ("Saturday");  
			}
			
			return "";
		}
		
		
		static private bool IsDayEmpty (int id)
		{
			foreach (string day in data [id])
				if (day != "")
					return false;
			
			return true;
		}
		
		
		static private string GetTable ()
		{
			string table = "	\n<table>\n";
			
			bool[] days = {IsDayEmpty (1), IsDayEmpty (2), IsDayEmpty (3),
							IsDayEmpty (4), IsDayEmpty (5), IsDayEmpty (6)};
			
			for (int i = 0; i < data [0].Count; i++) {
				if (i == 0) {
					table += "\t<thead>\n";
					table += "\t\t<tr>\n";
					table += "\t\t\t<th>" + ColumnText (0) + "</th>\n";
					
					if (days [0] == false)
						table += "\t\t\t<td>" + ColumnText (1) + "</td>\n";
					
					if (days [1] == false)
						table += "\t\t\t<td>" + ColumnText (2) + "</td>\n";
					
					if (days [2] == false)
						table += "\t\t\t<td>" + ColumnText (3) + "</td>\n";
					
					if (days [3] == false)
						table += "\t\t\t<td>" + ColumnText (4) + "</td>\n";
					
					if (days [4] == false)
						table += "\t\t\t<td>" + ColumnText (5) + "</td>\n";
					
					if (days [5] == false)
						table += "\t\t\t<td>" + ColumnText (6) + "</td>\n";
					
					table += "\t\t</tr>\n";
					table += "\t</thead>\n";
					table += "\t<tbody>\n";
				}
				
				if (i == (data [0].Count - 1)) {
					table += "\t</tbody>\n";
					table += "\t<tfoot>\n";
					table += "\t\t<tr>\n";
					table += "\t\t\t<th>" + new Time ((string) data [0][i], true).ToString (Settings.GetBool ("TimeMode24")) + "</th>\n";
					
					if (days [0] == false)
						table += "\t\t\t<td>" + data [1][i] + "</td>\n";
					
					if (days [1] == false)
						table += "\t\t\t<td>" + data [2][i] + "</td>\n";
					
					if (days [2] == false)
						table += "\t\t\t<td>" + data [3][i] + "</td>\n";
					
					if (days [3] == false)
						table += "\t\t\t<td>" + data [4][i] + "</td>\n";
					
					if (days [4] == false)
						table += "\t\t\t<td>" + data [5][i] + "</td>\n";
					
					if (days [5] == false)
						table += "\t\t\t<td>" + data [6][i] + "</td>\n";
					
					table += "\t\t</tr>\n";
					table += "\t</tfoot>\n";
				} else {
					table += "\t\t<tr>\n";
					table += "\t\t\t<th>" + new Time ((string) data [0][i], true).ToString (Settings.GetBool ("TimeMode24")) + "</th>\n";
					
					if (days [0] == false)
						table += "\t\t\t<td>" + data [1][i] + "</td>\n";
					
					if (days [1] == false)
						table += "\t\t\t<td>" + data [2][i] + "</td>\n";
					
					if (days [2] == false)
						table += "\t\t\t<td>" + data [3][i] + "</td>\n";
					
					if (days [3] == false)
						table += "\t\t\t<td>" + data [4][i] + "</td>\n";
					
					if (days [4] == false)
						table += "\t\t\t<td>" + data [5][i] + "</td>\n";
					
					if (days [5] == false)
						table += "\t\t\t<td>" + data [6][i] + "</td>\n";
					
					table += "\t\t</tr>\n";
				}
			}
			
			table += "	</table>\n";
			
			
			return table;
		}
		
		
		static private void MakeHTML (string template, string title)
		{
			// reading file
			StreamReader file = new StreamReader (template, System.Text.Encoding.UTF8);
			string content    = file.ReadToEnd ();
			file.Close ();
			
			
			// creating lessons schedule
			content = content.Replace ("[[title]]", title);
			content = content.Replace ("[[generator]]", "GNUTU " + gnutu.Defines.Version);
			content = content.Replace ("[[schedule]]", GetTable ());
			
			
			// writing file
			StreamWriter file2 = new StreamWriter (template, false, System.Text.Encoding.UTF8);
			file2.Write (content);
			
			file2.Close ();
		}
		
		
		static private void CopyTemplateToNewDirectory (string template, string path)
		{
			if (System.IO.Directory.Exists (path) == false)
			    System.IO.Directory.CreateDirectory (path);
			
			foreach (string file in System.IO.Directory.GetFiles (template))
			    if ((File.Exists (Path.Combine (path, Path.GetFileName (file))) == false) && (Path.GetFileName (file) != "info.xml"))
				File.Copy (file, Path.Combine (path, Path.GetFileName (file)));
			
			foreach (string directory in System.IO.Directory.GetDirectories (template))
			    CopyTemplateToNewDirectory (directory, Path.Combine (path, System.IO.Path.GetFileName (directory)));
		}


		static public string SkinName (string InfoFile)
		{
		    StreamReader reader;
		    
		    try {
		    	reader = new StreamReader (InfoFile, System.Text.Encoding.UTF8);
		    } catch  {
		    	return "";
		    }
		    
		    XmlTextReader read = new XmlTextReader (reader);
		    
		    try {
		    	while (read.Read ())
		    	    if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement))
		    		if ((read.Name == "name") && (read.NodeType == XmlNodeType.Element)) 
				    return read.ReadString ();
		    } catch {
			reader.Close ();
			return "";
		    }
		  
		    
		    reader.Close ();
		    return "";
		}


		static public ArrayList SkinsList (string directory, ArrayList list)
		{
		    if (System.IO.Directory.Exists (directory) == false)
			return list;
		   
		    bool cnte;
		    
		    foreach (string dn in System.IO.Directory.GetDirectories (directory)) {
			if (File.Exists (Path.Combine (Path.Combine (directory, dn), "template.html")) == false)
			    continue;

			string name = SkinName (Path.Combine (Path.Combine (directory, dn), "info.xml"));
			
			if (name != "") {
			    cnte = false;
			    
			    foreach (Skin tmp in list) {
			        if (tmp.name == name) {
				    cnte = true;
				    break;
				}
			    }

			    if (cnte == true)
				continue;
			    
			    Skin sk = new Skin ();
			    sk.name = name;
			    sk.path = Path.Combine (directory, dn);
			    list.Add (sk);
			}
		    }


		    return list;
		}
	
		
		static public int ExportToHTML (string template, string path, string title)
		{
			if (System.IO.File.Exists (template + FilesPaths.Separator () + "template.html") == false)
			    return 1;
			
			
			CopyTemplateToNewDirectory (template, path);
			MakeHTML (Path.Combine (path, "template.html"), title);
			File.Move (Path.Combine (path, "template.html"), Path.Combine (path, "index.html"));

			// remove backup files
			if (File.Exists (Path.Combine (path, "template.html~")))
				File.Delete (Path.Combine (path, "template.html~"));

			if (File.Exists (Path.Combine (path, "style.css~")))
				File.Delete (Path.Combine (path, "style.css~"));

			
			return 0;
		}
	}
}
