/*
 *  Settings.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Collections;
using System.Xml;
using System.IO;


namespace gnutu {
	public struct Option {
		public string key;
		public string val;
	}
	

	/*
	 * Settings class
	 */
	
	public class Settings {
		static ArrayList Data;
		
		
		static public bool Open ()
		{
			Data = new ArrayList ();
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.SettingsFile (), System.Text.Encoding.UTF8);
			} catch  {
				LoadDefaultSettings ();
				return false;
			}
			
			XmlTextReader read = new XmlTextReader (reader);
			Option        opt  = new Option ();
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						if ((read.Name == "option") && (read.NodeType == XmlNodeType.Element)) {
							opt     = new Option ();
							opt.key = read.GetAttribute ("key");
							opt.val = read.ReadString ();

							Data.Add (opt);
						}
					}
				}
			} catch {
				LoadDefaultSettings ();
				reader.Close ();
				return false;
			}
			

			reader.Close ();
			return true;
		}


		static public void Close ()
		{
			// save settings
			XmlTextWriter f = new XmlTextWriter
				(FilesPaths.SettingsFile (), System.Text.Encoding.UTF8);

			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "gnutusettings", "http://www.gnutu.org");
				foreach (Option opt in Data) {
					f.WriteStartElement ("option");
					f.WriteAttributeString ("key", opt.key);
						f.WriteString (opt.val);
					f.WriteEndElement ();
				}
			f.WriteEndElement ();

			f.Close ();
		}
		
		
		static public string Get (string key)
		{
			foreach (Option opt in Data) {
				if (opt.key == key)
					return opt.val;
			}

			return "";
		}
		
		
		static public bool GetBool (string key)
		{
			if (Get (key) == "")
				return true;			
			
			return System.Convert.ToBoolean (Get (key));
		}
		
		
		static public bool GetBoolDF (string key)
		{
			if (Get (key) == "")
				return false;			
			
			return System.Convert.ToBoolean (Get (key));
		}
		
		
		static public void Set (string key, string val)
		{
			int i = 0;
			
			foreach (Option opt in Data) {
				if (opt.key == key) {
					Option newopt = opt;
					newopt.val = val;
					Data [i] = newopt;
					return;
				}

				i++;
			}


			Option nopt = new Option ();
			nopt.key    = key;
			nopt.val    = val;
			
			Data.Add (nopt);

			return;
		}
		
		
		static public void SetBool (string key, bool val)
		{
			Set (key, val.ToString ());
		}
		
		
		static public void LoadDefaultSettings ()
		{
			int year2 = DateTime.Today.Year + 1;

			Set ("SchoolYearBegin", ("1-9-" + DateTime.Today.Year.ToString ()));
			Set ("SchoolYearEnd",   ("25-6-" + year2.ToString ()));
			Set ("HolidayEnd",   ("31-8-" + year2.ToString ()));
			Set ("LessonsStartAt", "08:00");
			SetBool ("MinimalizeWindowToTray", true);
			SetBool ("ShowOnlyTrayIcon", false);
			SetBool ("AutomaticSubjectsInserting", true);
			SetBool ("ShowToolbar", true);
			SetBool ("TimeMode24", true);
			SetBool ("DateFirstYear", false);
			SetBool ("ShowTestAndQuizes", true);
			SetBool ("ShowNotes", true);
			SetBool ("ShowToDoList", true);
			SetBool ("ShowCompletedTasks", false);
			SetBool ("ShowDiary", true);
			SetBool ("ShowLessonsList", true);
			SetBool ("ShowDaysLeft", true);
			SetBool ("ShowNearestTests", true);
			Set     ("PlusValue", "0,33");
			Set     ("MinusValue", "-0,33");
			Set     ("SubjectsList", "");
			SetBool ("InformationsAboutUpdates", true);
		}
	}
}
