/*
 *  db.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 *
 *  GNUTU Database
 */


using System;
using System.Collections;
using System.Xml;
using System.IO;
using Mono.Unix;


namespace gnutu {
	public enum Test {
		none,
		quiz,
		test
	}
	
	public enum Priority {
		high = 2,
		medium = 1,
		low = 0
	}
	
	public struct Date {
		public int day;
		public int month;
		public int year;
		
		public Date (DateTime date)
		{
			day   = date.Day;
			month = date.Month;
			year  = date.Year;
		}
		
		
		public Date (string date, bool FirstYear)
		{
			if (date == "") {
				day   = DateTime.Today.Day;
				month = DateTime.Today.Month;
				year  = DateTime.Today.Year;
			} else {
				string [] spl = date.Split ('-');
				
				if (FirstYear == true) {
					day   = System.Convert.ToInt32 (spl [2]);
					month = System.Convert.ToInt32 (spl [1]);
					year  = System.Convert.ToInt32 (spl [0]);
				} else {
					day   = System.Convert.ToInt32 (spl [0]);
					month = System.Convert.ToInt32 (spl [1]);
					year  = System.Convert.ToInt32 (spl [2]);
				}
			}
		}
		

		public string ToString (bool DateFirstYear)
		{
			if (DateFirstYear)
				return year.ToString () + "-" + month.ToString () + "-" + day.ToString ();
			else
				return day.ToString () + "-" + month.ToString () + "-" + year.ToString ();
		}
		
		
		public DateTime ToDateTime ()
		{
			return new DateTime (year, month, day);
		}
	}
	
	public struct Time {
		public int hour;
		public int minute;
		

		public Time (string time, bool Time24Mode)
		{
		    try {
			if (Time24Mode) {
				string [] spl = time.Split (':');
				
				hour   = System.Convert.ToInt32 (spl [0]);
				minute = System.Convert.ToInt32 (spl [1]);
			} else {
				string    ntime = time.Replace (" A.M.", "");
				string [] spl;
				
				if (ntime != time) { // if A.M.
					spl = ntime.Split (':');
					hour   = System.Convert.ToInt32 (spl [0]);
				} else {
					ntime = time.Replace (" P.M.", "");
					spl   = ntime.Split (':');
					hour   = System.Convert.ToInt32 (spl [0]) + 12;
				}
				
				minute = System.Convert.ToInt32 (spl [1]);
			}
		    } catch {
			hour   = 0;
			minute = 0;
		    }
		}
		
		
		public Time (DateTime time)
		{
			hour   = time.Hour;
			minute = time.Minute;
		}
		
		
		public string ToString (bool Time24Mode)
		{
			string sminute = minute.ToString ();
			if (minute <= 9)
				sminute = "0" + minute.ToString ();
			
			if (Time24Mode) {
				string shour = hour.ToString ();
				if (hour <= 9)
					shour = "0" + shour;
					
				return shour + ":" + sminute;
			} else {
				int    nhour;
				string shour;
				string mod;
				
				if (hour > 12) {
					mod = "P.M.";
					nhour = hour - 12;
				} else {
					mod = "A.M.";
					nhour = hour;
				}
				
				shour = nhour.ToString ();
				if (nhour <= 9)
					shour = "0" + shour;
				
				return shour + ":" + sminute + " " + mod;
			}
		}
		
		
		public DateTime ToDateTime ()
		{
			return new DateTime (2005, 8, 20, this.hour, this.minute, 0);
		}
	}
	
	public struct Lessons {
		public Time     time;
		public string   name;
		public string   marks;
		public Test     mod;
		public string   testdesc;
	}
	
	public struct ToDo {
		public string   item;
		public Priority priority;
		public bool     did;
		public bool     Reminder;
		public Time     ReminderAt;
		public int      ReminderCount;
	}
	
	public struct Entry {
		public Date      date;
		public ArrayList lessons;
		public ArrayList todo;
		public string    notes;
		public string    diary;
		

		public bool IsEmpty ()
		{
			if ((lessons.Count == 0) && (todo.Count == 0) && (notes == "") && (diary == ""))
				return true;
			else
				return false;
		}
	}
	
	public enum SearchCategory {
		notes,
		tests,
		tasks,
		diary
	}
	
	public struct SearchResult {
		public Date           date;
		public SearchCategory category;
		public int            score;
		
		
		public string CategoryToString ()
		{
			switch (category) {
				case SearchCategory.notes:  return Catalog.GetString ("Notes");
				case SearchCategory.diary:  return Catalog.GetString ("Diary");
				case SearchCategory.tasks:  return Catalog.GetString ("Tasks");
				case SearchCategory.tests:  return Catalog.GetString ("Tests and quizes");
			}
			
			return "";
		}
	}

	public struct SearchTestsResult {
		public Date   date;
		public string name;
		public string mod;
		public string desc;
	}
	
	public struct StatsResult {
		public string    subject;
		public ArrayList marks;
		public ArrayList marks2;
		public float     average;
		
		public StatsResult (string subject)
		{
			this.subject = subject;
			this.marks   = new ArrayList ();
			this.marks2  = new ArrayList ();
			this.average = 0;
		}
		
		
		public string MarksToString ()
		{
			string res = "";
			
			foreach (float mark in marks) {
				if (res == "")
					res = mark.ToString ();
				else
					res += ", " + mark.ToString ();
			}
			
			return res;
		}
		
		
		public string Marks2ToString ()
		{
			string res = "";
			
			foreach (string mark in marks2) {
				if (res == "")
					res = mark;
				else
					res += ", " + mark	;
			}
			
			return res;
		}
	}
	
	
		
	public class DB {
		static private ArrayList Data;
		static public  ArrayList SearchArray;
		
		
		static public bool Open ()
		{
			// load database
			Data = new ArrayList ();
			
			StreamReader reader;
			
			try {
				reader = new StreamReader (FilesPaths.DbFile (), System.Text.Encoding.UTF8);
			} catch  {
				return false;
			}
			
			XmlTextReader read   = new XmlTextReader (reader);
			Entry         ent    = new Entry ();
			Lessons       les    = new Lessons ();
			ToDo          todo   = new ToDo ();
			
			try {
				while (read.Read ()) {
					if ((read.NodeType == XmlNodeType.Element) || (read.NodeType == XmlNodeType.EndElement)) {
						// reading date
						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.Element)) {
							ent = new Entry ();
							
							ent.date    = new Date (read.GetAttribute ("date"), false);
							ent.notes   = "";
							ent.diary   = "";
							ent.lessons = new ArrayList ();
							ent.todo    = new ArrayList ();
							
							continue;
						}
						
						// reading notes
						if (read.Name == "notes") {
							ent.notes = read.ReadString ();
							continue;
						}
						
						// reading diary
						if (read.Name == "diary") {
							ent.diary = read.ReadString ();
							continue;
						}
						
						// reading lessons
						if ((read.Name == "lesson") && (read.NodeType == XmlNodeType.Element)) {
							les = new Lessons ();
							les.mod  = StringToTest (read.GetAttribute ("mod"));
							les.time = new Time (read.GetAttribute ("time"), true);
							continue;
						}
						
						if (read.Name == "name") {
							les.name = read.ReadString ();
							continue;
						}
						
						if (read.Name == "marks") {
							les.marks = read.ReadString ();
							continue;
						}
						
						if (read.Name == "description") {
							les.testdesc = read.ReadString ();
							continue;
						}
						
						if ((read.Name == "lesson") && (read.NodeType == XmlNodeType.EndElement)) {
							ent.lessons.Add (les);
							continue;
						}
						
						// reading todo list
						if ((read.Name == "task") && (read.NodeType == XmlNodeType.Element)) {
							todo = new ToDo ();
							
							todo.did           = Convert.ToBoolean (read.GetAttribute ("did"));
							todo.priority      = StringToPriority (read.GetAttribute ("priority"));
							todo.Reminder = Convert.ToBoolean (read.GetAttribute ("reminder"));
							
							if (read.GetAttribute ("remat") != null)
								todo.ReminderAt = new Time (read.GetAttribute ("remat"), true);
							
							todo.ReminderCount = Convert.ToInt32 (read.GetAttribute ("remcount"));
							 
							todo.item          = read.ReadString ();
							ent.todo.Add (todo);
							
							continue;
						}
						
						// Appending entry to database
						if ((read.Name == "entry") && (read.NodeType == XmlNodeType.EndElement)) {
							Data.Add (ent);
							continue;
						}
					}
				}
			} catch {
				reader.Close ();
				return false;
			}
			
			reader.Close ();
			return true;
		}
		
		
		static public bool Close ()
		{
			// save database
			XmlTextWriter f = new XmlTextWriter (FilesPaths.DbFile (), System.Text.Encoding.UTF8);
			f.Formatting = Formatting.Indented;
			f.WriteStartDocument ();
			f.WriteStartElement (null, "gnutudb", "http://www.gnutu.org");
			
			foreach (Entry e in Data) {
				f.WriteStartElement ("entry");
				f.WriteAttributeString ("date", e.date.ToString (false));
					f.WriteStartElement ("notes");
						f.WriteString (e.notes);
					f.WriteEndElement ();
					
					f.WriteStartElement ("lessons");
						
						foreach (Lessons l in e.lessons) {
							f.WriteStartElement ("lesson");
							f.WriteAttributeString ("mod", TestToString (l.mod));
							f.WriteAttributeString ("time", l.time.ToString (true));
								f.WriteStartElement ("name");
									f.WriteString (l.name);
								f.WriteEndElement ();
								
								f.WriteStartElement ("marks");
									f.WriteString (l.marks);
								f.WriteEndElement ();
								
								f.WriteStartElement ("description");
									f.WriteString (l.testdesc);
								f.WriteEndElement ();
							f.WriteEndElement ();
						}
					
					f.WriteEndElement ();
					f.WriteStartElement ("todo");
					
						foreach (ToDo t in e.todo) {
							f.WriteStartElement ("task");
							f.WriteAttributeString ("priority", PriorityToString (t.priority));
							f.WriteAttributeString ("did", t.did.ToString ());
							f.WriteAttributeString ("reminder", t.Reminder.ToString ());
							f.WriteAttributeString ("remat", t.ReminderAt.ToString (true));
							f.WriteAttributeString ("remcount", t.ReminderCount.ToString ());
								f.WriteString (t.item);
							f.WriteEndElement ();
						}
						
					f.WriteEndElement ();
					
					f.WriteStartElement ("diary");
						f.WriteString (e.diary);
					f.WriteEndElement ();
				f.WriteEndElement ();
			}
			
			f.WriteEndElement ();
			f.WriteEndDocument ();
			f.Close ();
			
			return true;
		}
		
		
		static public void AddEntry (Entry e)
		{
			Data.Add (e);
		}
		
		
		static public void AddEntry (Entry e, DateTime date)
		{
			e.date = new Date (date);
			Data.Add (e);
		}
		
		
		static public void ModifyEntry (Entry e, DateTime date)
		{
			e.date = new Date (date);
			
			int i = 0;
			foreach (Entry tmp in Data) {
				if ((tmp.date.day == e.date.day) && (tmp.date.month == e.date.month) &&
					    (tmp.date.year == e.date.year)) {
					Data [i] = e;
					return;
				}
				
				i++;
			}
		}
		
		
		static public bool EntryExists (DateTime date)
		{
			Date tmp = new Date (date);
			
			foreach (Entry e in Data) {
				if ((tmp.day == e.date.day) && (tmp.month == e.date.month) &&
					(tmp.year == e.date.year))
					return true;
			}
			
			return false;
		}
		
		
		static public Entry GetEntryByDate (DateTime date)
		{
			Date tmp = new Date (date);
			
			foreach (Entry e in Data) {
				if ((tmp.day == e.date.day) && (tmp.month == e.date.month) &&
					(tmp.year == e.date.year))
					return e;
			}
			
			return new Entry ();
		}
		
		
		static public ArrayList GetLessonsFromEntry (Entry entry)
		{
			return new ArrayList ();
		}
		
		
		static public ArrayList Search (string query, 
							bool notes, bool tests, bool tasks, bool diary,
							DateTime from, DateTime to, bool all)
		{
			ArrayList resarray = new ArrayList ();
			int       count;
			
			
			foreach (Entry e in Data) {
				if ((all == false) && ((e.date.ToDateTime () < from) || (e.date.ToDateTime () > to)))
						continue;
				
				
				// searching in notes
				if (notes == true) {
					count = SearchString (e.notes, query);
					
					if (count > 0) {
						SearchResult res = new SearchResult ();
						res.date = e.date;
						res.category = SearchCategory.notes;
						res.score = count;
						resarray.Add (res);
					}
				}
				
				
				// searching in lessons (test and quizes)
				if (tests == true) {
					count = 0;
					
					foreach (Lessons les in e.lessons) {
						if (les.mod == Test.quiz)
							count += SearchString (les.name, query) + SearchString (les.testdesc, query);
						else if (les.mod == Test.test)
							count += SearchString (les.name, query) + SearchString (les.testdesc, query);
					}
					
					if (count > 0) {
						SearchResult res = new SearchResult ();
						res.date = e.date;
						res.category = SearchCategory.tests;
						res.score = count;
						resarray.Add (res);
					}
				}
				
				
				// searching in ToDo list (tasks)
				if (tasks == true) {
					count = 0;
					
					foreach (ToDo task in e.todo)
						count += SearchString (task.item, query);
					
					if (count > 0) {
						SearchResult res = new SearchResult ();
						res.date = e.date;
						res.category = SearchCategory.tasks;
						res.score = count;
						resarray.Add (res);
					}
				}
				
				
				// search in diary
				if (diary == true) {
					SearchString (e.diary, query);
					count = SearchString (e.diary, query);
					
					if (count > 0) {
						SearchResult res = new SearchResult ();
						res.date = e.date;
						res.category = SearchCategory.diary;
						res.score = count;
						resarray.Add (res);
					}
				}
			}
			
			
			// search max and min value
			int min = 0;
			int max = 0;
			foreach (SearchResult res in resarray) {
				if ((min == 0) && (max == 0)) {
					min = res.score;
					max = res.score;
				} else if (min > res.score)
					min = res.score;
				else if (max < res.score)
					max = res.score;
			}
			
			
			// creating scores list
			float [] scores = {0, 0, 0, 0, 0};
			float r = ((float) max - (float) min) / 5;
			
			for (int i = 0; i < 4; i++)
				scores [i] = r * (i + 1);
			
			scores [4] = max;

			SearchArray = new ArrayList ();
			foreach (SearchResult res in resarray) {
				for (int i = 0; i <= 4; i++) {
					if ((float) res.score <= (float) scores [i]) {
						SearchResult nres = res;
						nres.score = i + 1;
						SearchArray.Add (nres);
						break;
					}
				}
			}
			
			
			SearchArray.Sort (new SearchResultComparer ());
			
			return SearchArray;
		}
		
		
		internal class SearchResultComparer : System.Collections.IComparer {
			public int Compare (object x, object y)
			{
				SearchResult a = (SearchResult) x;
				SearchResult b = (SearchResult) y;
				
				if (a.score > b.score)
					return -1;
				else if (a.score == b.score)
					return 0;
				else
					return 1;
			}
		}


		internal class SearchTestsResultComparer : System.Collections.IComparer {
			public int Compare (object x, object y)
			{
				SearchTestsResult a = (SearchTestsResult) x;
				SearchTestsResult b = (SearchTestsResult) y;
				
				if (a.date.ToDateTime () > b.date.ToDateTime ())
					return 1;
				else if (a.date.ToDateTime () == b.date.ToDateTime ())
					return 0;
				else
					return -1;
			}
		}

		

		static public ArrayList SearchTests (DateTime from, DateTime to, bool tests, bool quizes)
		{
			ArrayList res = new ArrayList ();

			foreach (Entry e in Data) {
				foreach (Lessons les in e.lessons) {
					if ((e.date.ToDateTime () >= from) && (e.date.ToDateTime () <= to)) {
						if ((tests == true) && (les.mod == Test.test)) {
							SearchTestsResult str = new SearchTestsResult ();
							str.date = e.date;
							str.name = les.name;
							str.mod  = Catalog.GetString ("Test");
							str.desc = les.testdesc;

							res.Add (str);
						}
	
						if ((quizes == true) && (les.mod == Test.quiz)) {
							SearchTestsResult str = new SearchTestsResult ();
							str.date = e.date;
							str.name = les.name;
							str.mod  = Catalog.GetString ("Quiz");
							str.desc = les.testdesc;

							res.Add (str);
						}
					}
				}
			}

			
			res.Sort (new SearchTestsResultComparer ());
			
			return res;
		}
		
		
		static public ArrayList CreateStats (bool IncludeMod, DateTime SearchFrom, DateTime SearchTo)
		{
			// create subjects list
			ArrayList subjects = new ArrayList ();
			
			foreach (Entry e in Data) {
				if ((e.date.ToDateTime () < SearchFrom) || (e.date.ToDateTime () > SearchTo))
					continue;
				
				foreach (Lessons les in e.lessons)
					subjects = AddSubject (subjects, les.name, les.marks, IncludeMod, (float) Convert.ToDecimal (Settings.Get ("PlusValue")), (float) Convert.ToDecimal (Settings.Get ("MinusValue")));
			}
			
			
			// average
			ArrayList subjects2 = new ArrayList ();
			foreach (StatsResult res in subjects) {
				if (res.marks.Count > 0) {
					StatsResult nres = res;
					nres.average = Average (res.marks);
					subjects2.Add (nres);
				}
			}
			
			
			return subjects2;
		}
		
		
		static private ArrayList GetMarksList (ArrayList list, string marks, bool IncludeMod, float PlusValue, float MinusValue)
		{
			string tmp = "";
			float  mark2 = 0; 
			
			for (int i = 0; i < marks.Length; i++) {
				if ((Char.IsDigit (marks [i]) == true))
					tmp += marks [i].ToString ();
				else {
					if ((IncludeMod) && (tmp != "")) {
						if (marks [i] == '+') {
							mark2 = (float) Convert.ToDecimal (tmp) + PlusValue;
							tmp = mark2.ToString ();
						}
						
						if (marks [i] == '-') {
							mark2 = (float) Convert.ToDecimal (tmp) + MinusValue;
							tmp = mark2.ToString ();
						}
					}
					
					if (tmp != "") {
						list.Add ((float) Convert.ToDecimal (tmp));
						tmp = "";
					}
				}
			}
			
			
			if (tmp != "")
				list.Add ((float) Convert.ToInt32 (tmp));
			
			
			return list;
		}
		
		
		static private ArrayList GetMarks2List (ArrayList list, string marks, bool IncludeMod)
		{
			string tmp = "";
			
			for (int i = 0; i < marks.Length; i++) {
				if (((Char.IsDigit (marks [i]) == true)) || (marks [i] == '+') || (marks [i] == '-')) {
					if ((marks [i] == '+') || (marks [i] == '-'))
						if (tmp == "")
							continue;
					
					tmp += marks [i].ToString ();
				} else {
					if (tmp != "") {
						list.Add (tmp);
						tmp = "";
					}
				}
			}
			
				
			if (tmp != "")
				list.Add (tmp);
			
			
			return list;
		}
		
		
		static private ArrayList AddSubject (ArrayList subjects, string subject, string marks, bool IncludeMod, float PlusValue, float MinusValue)
		{
			StatsResult item;
			int         i = 0;
			
			foreach (StatsResult subj in subjects) {
				if (subj.subject == subject) {
					item = (StatsResult) subjects [i];
					item.marks   = GetMarksList  (item.marks, marks, IncludeMod, PlusValue, MinusValue);
					item.marks2  = GetMarks2List (item.marks2, marks, IncludeMod);
					subjects [i] = item;
					
					return subjects;
				}
				
				i++;
			}
			
			item = new StatsResult (subject);
			item.marks  = GetMarksList  (item.marks, marks, IncludeMod, PlusValue, MinusValue);
			item.marks2 = GetMarks2List (item.marks2, marks, IncludeMod);
			subjects.Add (item);
			
			return subjects;
		}
		
		
		static public float Average (ArrayList list)
		{
			if (list.Count == 0)
				return 0;
			
			float average = 0;
			
			foreach (float mark in list)
				average += mark;
			
			return (average / list.Count);
		}
		
		
		
		static private int SearchString (string str, string keyword)
		{
			str     = str.ToLower ();
			keyword = keyword.ToLower ();
			
			int count = 0;
			int i     = 0;
			
			while (true) {
				i = str.IndexOf (keyword, i);
				
				if (i == -1)
					return count;
				
				count++;
				i++;
			}
		}
		
		
		static public string TestToString (Test t)
		{
			switch (t) {
				case Test.none: return "none";
				case Test.quiz: return "quiz";
				case Test.test: return "test";
			}
			
			return "none";
		}
		
		
		static private Test StringToTest (string t)
		{
			if (t == "none")
				return Test.none;
			
			if (t == "test")
				return Test.test;
			
			if (t == "quiz")
				return Test.quiz;
			
			return Test.none;
		}
		
		
		static public string PriorityToString (Priority p)
		{
			switch (p) {
				case Priority.low   : return "low";
				case Priority.medium: return "medium";
				case Priority.high  : return "high";
			}
			
			return "medium";
		}
		
		
		static private Priority StringToPriority (string p)
		{
			if (p == "medium")
				return Priority.medium;
			
			if (p == "high")
				return Priority.high;
			
			if (p == "low")
				return Priority.low;
			
			return Priority.medium;
		}
	}
}
