/*
 *  DaysOffSchoolWindow.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using Gtk;
using Glade;
using gnutu.Widgets;
using Mono.Unix;


namespace gnutu.GUI {

	/*
	 *  DaysOffSchoolWindow class
	 */
	
	public class DaysOffSchoolWindow {
		Glade.XML gui;
		
		[Glade.Widget] public  Gtk.Window   Window;
		[Glade.Widget] private Gtk.Calendar DaysOffSchoolCalendar;
		[Glade.Widget] private Gtk.Label    Description;
		[Glade.Widget] private Gtk.Button   AddBtn;
		[Glade.Widget] private Gtk.Button   RemoveBtn;
		[Glade.Widget] private Gtk.Button   ModifyBtn;
		
		
		public DaysOffSchoolWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "DaysOffSchoolWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("DaysOffSchoolWindow");
			
			DaysOffSchoolCalendarMark ();
			
			if (DaysOffSchool.IsDayOffSchool (new Date (DaysOffSchoolCalendar.Date)) == true) {
				Description.Text = DaysOffSchool.GetDayDescription (new Date (DaysOffSchoolCalendar.Date));
				AddBtn.Sensitive    = false;
				ModifyBtn.Sensitive = true;
				RemoveBtn.Sensitive = true;
			} else {
				AddBtn.Sensitive    = true;
				ModifyBtn.Sensitive = false;
				RemoveBtn.Sensitive = false;
			}
		}
		
		
		public void on_SettingsWindow_delete_event (object o, DeleteEventArgs args) 
		{
			args.RetVal = false;
		}
		
		
		public void on_CloseBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void on_AddBtn_clicked (object o, EventArgs args)
		{
			ModifyDayOffSchoolWindow wdw = new ModifyDayOffSchoolWindow (DaysOffSchoolCalendar, Description, false);
			wdw.Window.Show ();
			
			RemoveBtn.Sensitive  = false;
		}
		
		
		public void on_AddRangeBtn_clicked (object o, EventArgs args)
		{
			ModifyDayOffSchoolWindow wdw = new ModifyDayOffSchoolWindow (DaysOffSchoolCalendar, Description, true);
			wdw.Window.Show ();
			
			RemoveBtn.Sensitive  = false;
		}
		
		
		public void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			DaysOffSchool.RemoveDayOffSchool (new Date (DaysOffSchoolCalendar.Date));
			DaysOffSchoolCalendar.UnmarkDay ((uint) new Date (DaysOffSchoolCalendar.Date).day);
			Description.Text = "";

			AddBtn.Sensitive    = true;
			ModifyBtn.Sensitive = false;
			RemoveBtn.Sensitive = false;
		}
		
		
		public void on_ModifyBtn_clicked (object o, EventArgs args)
		{
			ModifyDayOffSchoolWindow wdw = new ModifyDayOffSchoolWindow (DaysOffSchoolCalendar, Description, false);
			wdw.Window.Show ();
			
			RemoveBtn.Sensitive  = false;
		}
		
		
		public void on_DaysOffSchoolCalendar_day_selected (object o, EventArgs args)
		{
			if (DaysOffSchool.IsDayOffSchool (new Date (DaysOffSchoolCalendar.Date)) == true) {
				Description.Text    = DaysOffSchool.GetDayDescription (new Date (DaysOffSchoolCalendar.Date));
				AddBtn.Sensitive    = false;
				ModifyBtn.Sensitive = true;
				RemoveBtn.Sensitive = true;
			} else {
				Description.Text    = "";
				AddBtn.Sensitive    = true;
				ModifyBtn.Sensitive = false;
				RemoveBtn.Sensitive = false;
			}
		}
		
		
		public void on_DaysOffSchoolCalendar_day_selected_double_click (object o, EventArgs args)
		{
			on_ModifyBtn_clicked (null, null);
		}
		
		
		public void on_DaysOffSchoolCalendar_month_changed (object o, EventArgs args)
		{
			DaysOffSchoolCalendarMark ();
		}
		
		
		public void on_DaysOffSchoolCalendar_next_year (object o, EventArgs args)
		{
			DaysOffSchoolCalendarMark ();
		}
		
		
		public void on_DaysOffSchoolCalendar_prev_year (object o, EventArgs args)
		{
			DaysOffSchoolCalendarMark ();
		}
		
		
		public void DaysOffSchoolCalendarMark ()
		{
			DaysOffSchoolCalendar.ClearMarks ();
			
			foreach (DayOffSchool day in DaysOffSchool.data)
				if ((day.date.month == (DaysOffSchoolCalendar.Month + 1)) && (day.date.year == DaysOffSchoolCalendar.Year))
					DaysOffSchoolCalendar.MarkDay ((uint) day.date.day);
		}
	}

	
	/*
	 *  ModifyDayOffSchoolWindow class
	 */
	
	public class ModifyDayOffSchoolWindow {
		private Glade.XML gui;
		
		[Glade.Widget] public  Gtk.Window      Window;
		[Glade.Widget] private Gtk.Calendar    DaysOffSchoolCalendar;
		[Glade.Widget] private Gtk.Entry       Description;
		[Glade.Widget] private Gtk.Label       DescriptionLabel;
		[Glade.Widget] private Gtk.Label       FromLabel;
		[Glade.Widget] private Gtk.Frame       DateFrame;
		[Glade.Widget] private Gtk.CheckButton Overwrite;
		[Glade.Widget] private Gtk.HBox        FromHBox;
		[Glade.Widget] private Gtk.HBox        ToHBox;
		
		private DateEdit begin;
		private DateEdit end;
		
		private bool range;
		
		
		public ModifyDayOffSchoolWindow (Gtk.Calendar DaysOffSchoolCalendar, Gtk.Label DescriptionLabel, bool range)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ModifyDayOffSchoolWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("ModifyDayOffSchoolWindow");
			this.DaysOffSchoolCalendar = DaysOffSchoolCalendar;
			this.DescriptionLabel = DescriptionLabel;
			
			
			if (range == true) {
				this.begin = new DateEdit ();
				this.end   = new DateEdit ();
				FromHBox.Add (this.begin);
				ToHBox.Add   (this.end);
				
				DateFrame.Visible = true;
				this.range = true;
			} else {
				DateFrame.Visible = false;
				this.range = false;
				this.Description.Text = DaysOffSchool.GetDayDescription (new Date (DaysOffSchoolCalendar.Date));
			}
		}
		
		
		public void on_Description_activate (object o, EventArgs args)
		{
			on_OkBtn_clicked (null, null);
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			if (range == true) {
				DateTime date1 = begin.GetDate ();
				DateTime date2 = end.GetDate ();
				
				while (date1 <= date2) {
					if ((DaysOffSchool.IsDayOffSchool (new Date (date1))) && (Overwrite.Active == true))
						DaysOffSchool.SetDayOffSchool (new Date (date1), Description.Text);
					else
						DaysOffSchool.SetDayOffSchool (new Date (date1), Description.Text);
					
					if (date1.Month == DaysOffSchoolCalendar.Month + 1)
						DaysOffSchoolCalendar.MarkDay ((uint) date1.Day);
					
					date1 = date1.AddDays (1);
				}
			} else {
				DaysOffSchool.SetDayOffSchool (new Date (DaysOffSchoolCalendar.Date), Description.Text);
				DaysOffSchoolCalendar.MarkDay ((uint) DaysOffSchoolCalendar.Day);
				DescriptionLabel.Text = Description.Text;
			}

			DaysOffSchoolCalendar.SelectDay ((uint) DaysOffSchoolCalendar.Day);
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
	}
}
