/*
 *  FirstStart.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using System.Collections;
using System.IO;
using System.Xml;
using Gtk;
using Glade;
using gnutu.Widgets;
using Mono.Unix;


namespace gnutu.GUI {

	/*
	 *  FirstStartWindow class
	 */

	public class FirstStartWindow {
		Glade.XML gui;
		bool      NewUser;

		[Glade.Widget] public  Gtk.Window      Window;
		[Glade.Widget] private Gtk.Notebook    Pages;
		[Glade.Widget] private Gtk.HBox        NextBtnBox;
		[Glade.Widget] private Gtk.HBox        BackBtnBox;
		[Glade.Widget] private Gtk.Button      NextBtn;
		[Glade.Widget] private Gtk.Button      BackBtn;
		[Glade.Widget] private Gtk.Viewport    Title1Bg;
		[Glade.Widget] private Gtk.Viewport    Title2Bg;
		[Glade.Widget] private Gtk.Viewport    Title3Bg;
		[Glade.Widget] private Gtk.Viewport    Title4Bg;
		[Glade.Widget] private Gtk.Viewport    Title5Bg;
		[Glade.Widget] private Gtk.Image       TitleIcon1;
		[Glade.Widget] private Gtk.Image       TitleIcon2;
		[Glade.Widget] private Gtk.Image       TitleIcon3;
		[Glade.Widget] private Gtk.Image       TitleIcon4;
		[Glade.Widget] private Gtk.Image       TitleIcon5;
		[Glade.Widget] private Gtk.ProgressBar ImportingProgress;
		[Glade.Widget] private Gtk.Table       GeneralTable;
		[Glade.Widget] private Gtk.CheckButton TimeMode24;
		[Glade.Widget] private Gtk.CheckButton DateFirstYear;

		private Gtk.Image NextBtnImage;
		private Gtk.Label NextBtnLabel;
		private DateEdit  SchoolYearBegin;
		private DateEdit  SchoolYearEnd;
		private DateEdit  HolidayEnd;


		public FirstStartWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "FirstStartWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("FirstStartWindow");

			if (System.IO.File.Exists (FilesPaths.OldDbFile ()))
				NewUser = false;
			else
				NewUser = true;

			Pages.ShowTabs = false;
			Title1Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			Title2Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			Title3Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			Title4Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			Title5Bg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			TitleIcon1.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "logo-48.png");
			TitleIcon2.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Import-48.png");
			TitleIcon3.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Settings-48.png");
			TitleIcon4.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "LessonsSchedule-48.png");
			TitleIcon5.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "WizardFinish-48.png");


			// buttons
			NextBtnImage = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Next-20.png"));
			NextBtnLabel = new Gtk.Label (Catalog.GetString ("Next"));
			NextBtnBox.Add (NextBtnImage);
			NextBtnBox.Add (NextBtnLabel);
			NextBtnBox.ShowAll ();

			Gtk.Image BackBtnImage = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Previous-20.png"));
			Gtk.Label BackBtnLabel = new Gtk.Label (Catalog.GetString ("Back"));
			BackBtnBox.Add (BackBtnImage);
			BackBtnBox.Add (BackBtnLabel);
			BackBtnBox.ShowAll ();


			// general settings
			Settings.Open ();
			SchoolYearBegin = new DateEdit ();
			SchoolYearEnd   = new DateEdit ();
			HolidayEnd      = new DateEdit ();

			SchoolYearBegin.SetDate (new Date (Settings.Get ("SchoolYearBegin"), false).ToDateTime ());
			SchoolYearEnd.SetDate (new Date (Settings.Get ("SchoolYearEnd"), false).ToDateTime ());
			HolidayEnd.SetDate (new Date (Settings.Get ("HolidayEnd"), false).ToDateTime ());

			GeneralTable.Attach (SchoolYearBegin, 1, 2, 0, 1);
			GeneralTable.Attach (SchoolYearEnd, 1, 2, 1, 2);
			GeneralTable.Attach (HolidayEnd, 1, 2, 2, 3);


			// LessonsSchedule
			LessonsScheduleData.Open ();
			this.Window.ShowAll ();
		}


		public void on_FirstStartWindow_delete_event (object o, DeleteEventArgs args)
		{
			args.RetVal = true;
		}


		public void on_BackBtn_clicked (object o, EventArgs args)
		{
			if ((Pages.Page == 1) && (NewUser))
				Pages.Page--;

			if ((Pages.Page == 2) && (NewUser == true))
				Pages.Page--;

			if ((Pages.Page == 3) && (NewUser == false))
				Pages.Page--;

			if (Pages.Page > 0)
				Pages.Page--;

			if (Pages.Page == 3) {
				NextBtnLabel.Text = Catalog.GetString ("Back");
				Pages.Page--;
			}

			if (Pages.Page == 0)
				BackBtn.Sensitive = false;
		}


		public void on_NextBtn_clicked (object o, EventArgs args)
		{
			if (Pages.Page == 4) {
				this.Window.Sensitive = false;

				while (Application.EventsPending ())
					Application.RunIteration ();

				Settings.SetBool ("TimeMode24", TimeMode24.Active);
				Settings.SetBool ("DateFirstYear", DateFirstYear.Active);
				Settings.Set     ("SchoolYearBegin", SchoolYearBegin.GetDate2 ().ToString (false));
				Settings.Set     ("SchoolYearEnd", SchoolYearEnd.GetDate2 ().ToString (false));
				Settings.Set     ("HolidayEnd", HolidayEnd.GetDate2 ().ToString (false));
				Settings.Close ();

				if (NewUser == true)
					LessonsScheduleData.Close ();
				
				new MainWindow ();
				this.Window.Destroy ();
			}


			if ((Pages.Page == 0) && (NewUser))
				Pages.Page++;

			if ((Pages.Page == 2) && (NewUser == false))
				Pages.Page++;

			if (Pages.Page <= 4)
				Pages.Page++;

			if (Pages.Page == 4) {
				NextBtnLabel.Text = Catalog.GetString ("Finish");
				NextBtnImage.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Finish-20.png");
				Pages.Page++;
			}

			BackBtn.Sensitive = true;
		}


		public void on_LessonsScheduleBtn_clicked (object o, EventArgs args)
		{
			LessonsScheduleData.Open ();
			LessonsScheduleWindow wdw = new LessonsScheduleWindow ();
			wdw.Window.Show ();
		}


		public void on_Pages_switch_page (object o, SwitchPageArgs args)
		{
			if (Pages.Page == 1) {
				NextBtn.Sensitive = false;
				BackBtn.Sensitive = false;

				ImportDB ();
				ImportingProgress.Fraction = (float) 0.33;
				while (Application.EventsPending ())
                	Application.RunIteration ();

				ImportLessonsSchedule ();
				ImportingProgress.Fraction = (float) 0.66;
				while (Application.EventsPending ())
                	Application.RunIteration ();

				ImportDaysOffSchool ();
				ImportingProgress.Fraction = (float) 1.00;
				while (Application.EventsPending ())
                	Application.RunIteration ();

				NextBtn.Sensitive = true;
				BackBtn.Sensitive = true;
			}
		}


		public void ImportDB ()
		{
			StreamReader OldDb;

			try {
				OldDb = new StreamReader (FilesPaths.OldDbFile ());

				XmlTextWriter f = new XmlTextWriter (FilesPaths.DbFile (), System.Text.Encoding.UTF8);
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "gnutudb", "http://www.gnutu.org");

				Entry   entry   = new Entry ();
				Lessons lesson  = new Lessons ();
				string  line    = "";
				string  content = OldDb.ReadToEnd ();
				int     i       = 0;
				int     j       = 0;

				while (j < content.Length) {
					if (content [j] == '\n') {
						if (line.Length > 0) {
							if (line [0] == '.') {
								string [] tab = LineSplit (line);

								switch (Convert.ToInt32 (tab [0])) {
									case 1:
											i = 0;
											entry = new Entry ();
											entry.date    = new Date (tab [2], false);
											entry.todo    = new ArrayList ();
											entry.lessons = new ArrayList ();
											entry.diary   = ImportDiary (tab [2]);
											break;

									case 2:
											lesson = new Lessons ();
											lesson.name = tab [2];
											break;

									case 3:
											lesson.marks = tab [2];
											break;

									case 4:
											if (tab [2] == "2") {
												lesson.testdesc = Catalog.GetString ("No description");
												lesson.mod      = Test.test;
											} else if (tab [2] == "1") {
												lesson.testdesc = Catalog.GetString ("No description");
												lesson.mod      = Test.quiz;
											} else {
												lesson.testdesc = "";
												lesson.mod      = Test.none;
											}

											break;

									case 5:
											try {
												lesson.time = new Time (tab [2], true);
											} catch {
												try {
													lesson.time = new Time (tab [2].ToUpper (), false);
												} catch {
													i++;
													string time = i.ToString () + ":00";
													lesson.time = new Time (time, true);
												}
											}

											entry.lessons.Add (lesson);
											break;

									case 6:
											entry.notes = tab [2];

											f.WriteStartElement ("entry");
											f.WriteAttributeString ("date", entry.date.ToString (false));
												f.WriteStartElement ("notes");
													f.WriteString (entry.notes);
												f.WriteEndElement ();

												f.WriteStartElement ("lessons");

													foreach (Lessons l in entry.lessons) {
														f.WriteStartElement ("lesson");
														f.WriteAttributeString ("mod", DB.TestToString (l.mod));
														f.WriteAttributeString ("time", l.time.ToString (true));
															f.WriteStartElement ("name");
																f.WriteString (l.name);
															f.WriteEndElement ();

															f.WriteStartElement ("marks");
																f.WriteString (l.marks);
															f.WriteEndElement ();

															f.WriteStartElement ("description");
																f.WriteString (l.testdesc);
															f.WriteEndElement ();
														f.WriteEndElement ();
													}

												f.WriteEndElement ();
												f.WriteStartElement ("todo");

													foreach (ToDo t in entry.todo) {
														f.WriteStartElement ("task");
														f.WriteAttributeString ("priority", DB.PriorityToString (t.priority));
														f.WriteAttributeString ("did", t.did.ToString ());
															f.WriteString (t.item);
														f.WriteEndElement ();
													}

												f.WriteEndElement ();

												f.WriteStartElement ("diary");
													f.WriteString (entry.diary);
												f.WriteEndElement ();
											f.WriteEndElement ();

											break;
								}
							}
						}

						line = "";
					} else
						line += content [j];

					j++;
				}

				f.WriteEndElement ();
				f.WriteEndDocument ();
				f.Close ();
			} catch  {
				return;
			}
		}


		public void ImportLessonsSchedule ()
		{
			StreamReader OldDb;

			try {
				OldDb = new StreamReader (FilesPaths.OldLessonsScheduleFile ());

				XmlTextWriter f = new XmlTextWriter (FilesPaths.LessonsScheduleFile (), System.Text.Encoding.UTF8);
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "LessonsSchedule", "http://www.gnutu.org");

				string    line = "";
				int       i    = 0;
				int       j    = 0;
				ArrayList list = new ArrayList ();
				string [] tab  = new string [] {};

				string content = OldDb.ReadToEnd ();

				while (i < content.Length) {
					if (content [i] == '\n'){
						if (line.Length > 0) {
							if (line [0] == '.') {
								tab = LineSplit (line);

								if (tab [1] == "1") {
									if (tab [0] != "0") {
										f.WriteStartElement ("day");

										if (tab [0] == "1") {
											foreach (string day in list) {
												f.WriteStartElement ("subject");
													Time time;

													try {
														time = new Time (day, true);
													} catch {
														try {
															time = new Time (day.ToUpper (), false);
														} catch {
															j++;
															string time2 = j.ToString () + ":00";
															time = new Time (time2, true);
														}
													}

													f.WriteString (time.ToString (true));
												f.WriteEndElement ();
											}
										} else {
											foreach (string day in list) {
												f.WriteStartElement ("subject");
													f.WriteString (day);
												f.WriteEndElement ();
											}
										}

										f.WriteEndElement ();

										list = new ArrayList ();
									}
								}
								list.Add (tab [2]);

							}
						}

						line = "";
					} else
						line += content [i];

					i++;
				}


				f.WriteStartElement ("day");
					foreach (string day in list) {
						f.WriteStartElement ("subject");
							f.WriteString (day);
						f.WriteEndElement ();
					}
				f.WriteEndElement ();

				f.WriteEndElement ();
				f.WriteEndDocument ();
				f.Close ();
			} catch  {
				return;
			}
		}


		public void ImportDaysOffSchool ()
		{
			StreamReader OldDb;

			try {
				OldDb = new StreamReader (FilesPaths.OldDaysOffSchoolFile ());

				XmlTextWriter f = new XmlTextWriter (FilesPaths.DaysOffSchoolFile (), System.Text.Encoding.UTF8);
				f.Formatting = Formatting.Indented;
				f.WriteStartDocument ();
				f.WriteStartElement (null, "DaysOffSchool", "http://www.gnutu.org");

				string    line = "";
				int       i    = 0;
				string [] tab  = new string [] {};

				string content = OldDb.ReadToEnd ();

				while (i < content.Length) {
					if (content [i] == '\n'){
						if (line.Length > 0) {
							if (line [0] == '.') {
								tab = LessonsScheduleLineSplit (line);

								f.WriteStartElement ("day");
									f.WriteAttributeString ("date", new Date (tab [0], false).ToString (false));
									f.WriteString (tab [1]);
								f.WriteEndElement ();
							}
						}

						line = "";
					} else
						line += content [i];

					i++;
				}

				f.WriteEndElement ();
				f.WriteEndDocument ();
				f.Close ();
			} catch  {
				return;
			}
		}


		public string ImportDiary (string date)
		{
			if (File.Exists (FilesPaths.OldDiaryDirectory () + date + ".gtu")) {
				try {
					StreamReader diary = new StreamReader (FilesPaths.OldDiaryDirectory () + date + ".gtu");
					return diary.ReadToEnd ();
				} catch  {
					return "";
				}
			}

			return "";
		}


		private string [] LineSplit (string line)
		{
			string [] tmp = line.Split (new char [] {'.', '.', ' '}, 4);
			tmp [3] = tmp [3].Replace ('|', '\n');

			string [] list = {tmp [1], tmp [2], tmp [3]};
			return list;
		}


		private string [] LessonsScheduleLineSplit (string line)
		{
			string [] tmp = line.Split (new char [] {'.', ' '}, 3);

			string [] list = {tmp [1], tmp [2]};
			return list;
		}
	}
}
