/*
 *  LessonsScheduleWindow.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using System.Collections;
using System.IO;
using Gtk;
using Glade;
using gnutu.Widgets;
using Mono.Unix;


namespace gnutu.GUI {

	/*
	 * LessonsScheduleWindow class
	 */
	
	public class LessonsScheduleWindow {
		private Glade.XML gui;
		
		[Glade.Widget] public  Gtk.Window     Window;
		[Glade.Widget] private Gtk.TreeView   LessonsSchedule;
		[Glade.Widget] private Gtk.HButtonBox EditDaysBox;
		[Glade.Widget] private Gtk.Image      ExportToHTMLImg;
		
		
		public LessonsScheduleWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "LessonsScheduleWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("LessonsScheduleWindow");
			
			ExportToHTMLImg.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "export-html-20.png");
			
			
			// creating LessonsList
			ListStore store = new ListStore (typeof (string), typeof (string),
						typeof (string), typeof (string), typeof (string),
						typeof (string), typeof (string));
			
			LessonsSchedule.Model = store;
			
			LessonsSchedule.AppendColumn (Catalog.GetString ("Time"), new CellRendererText (), "text", 0);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Monday"), new CellRendererText (), "text", 1);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Tuesday"), new CellRendererText (), "text", 2);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Wednesday"), new CellRendererText (), "text", 3);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Thursday"), new CellRendererText (), "text", 4);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Friday"), new CellRendererText (), "text", 5);
			LessonsSchedule.AppendColumn (Catalog.GetString ("Saturday"), new CellRendererText (), "text", 6);
			
			store.SetSortFunc (0, new Gtk.TreeIterCompareFunc (LessonsScheduleCompare));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			
			// loading lessons schedule
			ArrayList [] arr = LessonsScheduleData.data;
			
			int i = 0;
			while (i < arr [0].Count) {
				store.AppendValues (new object [] {new Time ((string) arr [0][i], true).ToString (Settings.GetBool ("TimeMode24")),
										(string) arr [1][i], (string) arr [2][i],
										(string) arr [3][i], (string) arr [4][i], (string) arr [5][i], (string) arr [6][i]});
				
				i++;
			}
			

			if (i == 0) {
				EditDaysBox.Sensitive = false;
				MessageHours ();
			} else
				EditDaysBox.Sensitive = true;
		}
		
		
		private int LessonsScheduleCompare (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
		{
			Time t1 = new Time ((string) model.GetValue (a, 0), Settings.GetBool ("TimeMode24"));
			Time t2 = new Time ((string) model.GetValue (b, 0), Settings.GetBool ("TimeMode24"));
			
			if (t1.ToDateTime () > t2.ToDateTime ())
				return 1;
			else if (t1.ToDateTime () == t2.ToDateTime ())
				return 0;
			
			return -1;
		}
		
		
		public void on_LessonsScheduleWindow_delete_event (object o, DeleteEventArgs args) 
		{
			args.RetVal = false;
		}


		public void on_HoursBtn_clicked (object o, EventArgs args)
		{
			EditHoursWindow wdw = new EditHoursWindow (EditDaysBox, (Gtk.ListStore) LessonsSchedule.Model);
			wdw.Window.Show ();
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("MondayBtn");
		}


		public void on_MondayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 1);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("TuesdayBtn");
		}


		public void on_TuesdayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 2);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("WednesdayBtn");
		}


		public void on_WednesdayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 3);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("ThursdayBtn");
		}

		
		public void on_ThursdayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 4);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("FridayBtn");
		}


		public void on_FridayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 5);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("SaturdayBtn");
		}
		

		public void on_SaturdayBtn_clicked (object o, EventArgs args)
		{
			EditLessonsWindow wdw = new EditLessonsWindow ((Gtk.ListStore) LessonsSchedule.Model, 6);
			wdw.Window.Show ();
			
			this.Window.Focus = (Gtk.Button) gui.GetWidget ("OkBtn");
		}
		
		
		public void on_ExportToHTMLBtn_clicked (object o, EventArgs args)
		{
			ExportingLessonsScheduleWindow wdw = new ExportingLessonsScheduleWindow ();
			wdw.window.Show ();
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			Gtk.ListStore store = (Gtk.ListStore) this.LessonsSchedule.Model;
			Gtk.TreeIter  iter;
			
			ArrayList [] data = new ArrayList [] {new ArrayList (), new ArrayList (), new ArrayList (), 
													new ArrayList (), new ArrayList (), new ArrayList (),
													new ArrayList (), new ArrayList ()};
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				data[0].Add (new Time ((string) store.GetValue (iter, 0), Settings.GetBool ("TimeMode24")).ToString (true));
				data[1].Add ((string) store.GetValue (iter, 1));
				data[2].Add ((string) store.GetValue (iter, 2));
				data[3].Add ((string) store.GetValue (iter, 3));
				data[4].Add ((string) store.GetValue (iter, 4));
				data[5].Add ((string) store.GetValue (iter, 5));
				data[6].Add ((string) store.GetValue (iter, 6));
			}
			
			
			LessonsScheduleData.data = data;
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		private void MessageHours ()
		{
			Gtk.VBox WindowBox = (Gtk.VBox) gui.GetWidget ("WindowBox");
			
			Alert alert = new Alert (AlertType.Info,
					    Catalog.GetString ("Fill in the hours !"),
					    Catalog.GetString ("In the begining you should put in at what time each class starts."));
			WindowBox.Add (alert);
		}
	}



	/*
	 *  EditHoursWindow
	 */
	
	public class EditHoursWindow {
		private Glade.XML     gui;

		[Glade.Widget] public  Gtk.Window     Window;
		[Glade.Widget] private Gtk.TreeView   HoursList;

		private TimeEdit       Hour;
		private Gtk.ListStore  LessonsListStore;
		private Gtk.HButtonBox EditDaysBox;
		
		
		public EditHoursWindow (Gtk.HButtonBox EditDaysBox, Gtk.ListStore LessonsListStore)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "EditHoursWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("EditHoursWindow");

			this.LessonsListStore = LessonsListStore;
			this.EditDaysBox      = EditDaysBox;

			
			Gtk.HBox HourBox = (Gtk.HBox) gui.GetWidget ("HourBox");
			Hour = new TimeEdit ();
			HourBox.Add (Hour);
			HourBox.ShowAll ();

			Hour.Hour.Activated    += new EventHandler (on_Hour_activated);
			Hour.Minutes.Activated += new EventHandler (on_Hour_activated);
			
			Window.Focus = Hour.Hour;
			
			
			ListStore store = new ListStore (typeof (string));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			HoursList.Model = store;
			
			HoursList.AppendColumn (Catalog.GetString ("Hour"), new CellRendererText (), "text", 0);
			
			store.SetSortFunc (0, new Gtk.TreeIterCompareFunc (HoursListCompare));

			Gtk.TreeIter  iter;
			for (bool tmp = LessonsListStore.GetIterFirst (out iter); tmp == true; tmp = LessonsListStore.IterNext (ref iter))
				store.AppendValues (new object [] {(string) LessonsListStore.GetValue (iter, 0)});
		}
		
		
		private int HoursListCompare (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
		{
			Time t1 = new Time ((string) model.GetValue (a, 0), Settings.GetBool ("TimeMode24"));
			Time t2 = new Time ((string) model.GetValue (b, 0), Settings.GetBool ("TimeMode24"));
			
			if (t1.ToDateTime () > t2.ToDateTime ())
				return 1;
			else if (t1.ToDateTime () == t2.ToDateTime ())
				return 0;
			
			return -1;
		}


		public void on_Hour_activated (object o, EventArgs args)
		{
			on_AddBtn_clicked (null, null);
		}


		public void on_HoursList_cursor_changed (object o, EventArgs args)
		{
			Gtk.Button RemoveBtn = (Gtk.Button) gui.GetWidget ("RemoveBtn");
			
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (HoursList.Selection.GetSelected (out mod, out itr))
				RemoveBtn.Sensitive = true;
			else
				RemoveBtn.Sensitive = false;
		}
		

		public void on_AddBtn_clicked (object o, EventArgs args)
		{
			Gtk.ListStore store = (Gtk.ListStore) HoursList.Model;
			Gtk.TreeIter  iter;
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				if (Hour.GetText () == (string) store.GetValue (iter, 0))
					return;
			}


			store.AppendValues (new object [] {Hour.GetText ()});
			Window.Focus = Hour.Hour;

			Gtk.Button RemoveBtn = (Gtk.Button) gui.GetWidget ("RemoveBtn");
			RemoveBtn.Sensitive = false;
		}
		
		
		public void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (HoursList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) HoursList.Model;
				store.Remove (ref itr);

				Gtk.Button btn = (Gtk.Button) o;
				btn.Sensitive = false;
			}
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeIter  itr2;
			Gtk.ListStore store = (Gtk.ListStore) HoursList.Model;
			
			int  cnt  = 0;
			bool tmp2;


			// remove last rows
			bool tmp = store.GetIterFirst (out itr);
			
			for (tmp2 = LessonsListStore.GetIterFirst (out itr2); tmp2 == true; tmp2 = LessonsListStore.IterNext (ref itr2)) {
				if ((tmp == true) && (cnt > 0))
					tmp = store.IterNext (ref itr);
				
				if (tmp == false) {
					while (LessonsListStore.Remove (ref itr2))
						continue;
					
					break;
				}
				
				cnt++;
			}
			
			
			// change
			ArrayList iters = new ArrayList ();
			for (tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr))
				iters.Add (itr);
			
			cnt  = 0;
			tmp2 = true;
			
			for (tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr)) {
				if (cnt >= iters.Count)
					tmp2 = false;

				if (tmp2 == true)
					LessonsListStore.SetValue ((Gtk.TreeIter) iters [cnt], 0, (string) store.GetValue (itr, 0));
				else
					LessonsListStore.AppendValues (new object [] {(string) store.GetValue (itr, 0), "", "", "", "", "", ""});
				
				cnt++;
			}
			
			
			if (cnt == 0)
				EditDaysBox.Sensitive = false;
			else
				EditDaysBox.Sensitive = true;
			
			
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
	}



	/*
	 *  EditLessonsWindow class
	 */
	
	public class EditLessonsWindow {
		private Glade.XML     gui;

		private Gtk.ListStore LessonsListStore;
		private int day;

		[Glade.Widget] public  Gtk.Window   Window;
		[Glade.Widget] private Gtk.TreeView LessonsList;
		[Glade.Widget] private Gtk.ComboBox Hour;
		[Glade.Widget] private Gtk.Entry    Subject;
		[Glade.Widget] private Gtk.Button   AddBtn;
		[Glade.Widget] private Gtk.Button   RemoveBtn;
		
		
		public EditLessonsWindow (Gtk.ListStore LessonsListStore, int day)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "EditLessonsWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("EditLessonsWindow");
			
			this.LessonsListStore = LessonsListStore;
			this.day = day;
						
			
			ListStore store = new ListStore (typeof (string), typeof (string));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			LessonsList.Model = store;
			
			LessonsList.AppendColumn (Catalog.GetString ("Hour"),    new CellRendererText (), "text", 0);
			LessonsList.AppendColumn (Catalog.GetString ("Subject"), new CellRendererText (), "text", 1);
			
			store.SetSortFunc (0, new Gtk.TreeIterCompareFunc (HoursListCompare));
			
			
			Gtk.TreeIter iter;
			for (bool tmp = LessonsListStore.GetIterFirst (out iter); tmp == true; tmp = LessonsListStore.IterNext (ref iter)) {
				if ((string) LessonsListStore.GetValue (iter, day) != "")
					store.AppendValues (new object [] {(string) LessonsListStore.GetValue (iter, 0), (string) LessonsListStore.GetValue (iter, day)});
				else
					Hour.AppendText ((string) LessonsListStore.GetValue (iter, 0));
			}
			
			
			Gtk.EntryCompletion comp = new Gtk.EntryCompletion ();
			comp.TextColumn = 0;
			Subject.Completion = comp;
			comp.Model = CreateCompletion ();	
			
			
			Window.Focus = Subject;
			
			if (Hour.Model.IterNChildren () > 0)
				Hour.Active = 0;
	}
		
		
		private int HoursListCompare (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
		{
			Time t1 = new Time ((string) model.GetValue (a, 0), Settings.GetBool ("TimeMode24"));
			Time t2 = new Time ((string) model.GetValue (b, 0), Settings.GetBool ("TimeMode24"));
			
			if (t1.ToDateTime () > t2.ToDateTime ())
				return 1;
			else if (t1.ToDateTime () == t2.ToDateTime ())
				return 0;

			return -1;
		}


		public void on_LessonsList_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.Button == 3) && (Hour.Model.IterNChildren () > 0) && (LessonsList.Model.IterNChildren () > 0)) {
				Glade.XML m = Glade.XML.FromAssembly ("gui.glade", "EditLessonsMenu", null);
				m.Autoconnect (this);
				Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("EditLessonsMenu");
				
				menu.Popup ();
				
				Gtk.MenuItem move_top  = (Gtk.MenuItem) m.GetWidget ("move_top");
				Gtk.MenuItem move_down = (Gtk.MenuItem) m.GetWidget ("move_down");
				
				move_top.Sensitive  = CheckFreeHour (true);
				move_down.Sensitive = CheckFreeHour (false);
			}
		}


		private bool CheckFreeHour (bool Top)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;
			string        hour = "";
			string        BeginHour = "";
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (Top == true)
					BeginHour = (string) store.GetValue (itr, 0);
				else {
					BeginHour = (string) store.GetValue (itr, 0);
					break;
				}

			
			for (bool tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr)) {
				if (Top == false)
					if (BeginHour == (string) LessonsListStore.GetValue (itr, 0))
						return false;
					else
						return true;

				hour = (string) LessonsListStore.GetValue (itr, 0);
			}


			if (hour == BeginHour)
				return false;
			else
				return true;
		}


		private string CheckHour (string Hour, bool Next)
		{
			Gtk.TreeIter itr;
			string       CurrentHour = "";
			
			for (bool tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr)) {
				if (Hour == (string) LessonsListStore.GetValue (itr, 0))
					if (Next == true) {
						LessonsListStore.IterNext (ref itr);
						return (string) LessonsListStore.GetValue (itr, 0);
					} else
						return CurrentHour;
				
				CurrentHour = (string) LessonsListStore.GetValue (itr, 0);
			}

			return "";
		}
		

		private void HourRemove (string hour)
		{
			Gtk.ListStore store = (Gtk.ListStore) Hour.Model;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (hour == (string) store.GetValue (itr, 0)) {
					store.Remove (ref itr);
					return;
				}
		}


		private void UpdateHours ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store  = (Gtk.ListStore) LessonsList.Model;	
			Gtk.ListStore hstore = (Gtk.ListStore) Hour.Model;

			hstore.Clear ();
			for (bool tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr))
				Hour.AppendText ((string) LessonsListStore.GetValue (itr, 0));
			
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				HourRemove ((string) store.GetValue (itr, 0));
		}


		public void on_move_top_activate (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, 0, CheckHour ((string) store.GetValue (itr, 0), true));

			UpdateHours ();
		}


		public void on_move_down_activate (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;

			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, 0, CheckHour ((string) store.GetValue (itr, 0), false));

			UpdateHours ();
		}
		
		
		public void on_Hour_changed (object o, EventArgs args)
		{
			Gtk.TreeIter iter;
			Hour.GetActiveIter (out iter);
			
			string text = (string) Hour.Model.GetValue (iter, 0);
			
			if ((text.Length > 0) && (Subject.Text.Length > 0))
				AddBtn.Sensitive = true;
			else
				AddBtn.Sensitive = false;
		}
		
		
		public void on_Subject_changed (object o, EventArgs args)
		{
			Gtk.TreeIter iter;
			string      text = "";
			
			if (Hour.GetActiveIter (out iter))
				text = (string) Hour.Model.GetValue (iter, 0);
			
			if ((text.Length > 0) && (Subject.Text.Length > 0))
				AddBtn.Sensitive = true;
			else
				AddBtn.Sensitive = false;
		}
		
		
		public void on_LessonsList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (LessonsList.Selection.GetSelected (out mod, out itr))
				RemoveBtn.Sensitive = true;
			else
				RemoveBtn.Sensitive = false;
		}
		
		
		public void on_Subject_activate (object o, EventArgs args)
		{
			if (AddBtn.Sensitive)
				on_AddBtn_clicked (null, null);
		}
		
		
		public void on_AddBtn_clicked (object o, EventArgs args)
		{
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;
			
			Gtk.TreeIter  iter;
			Hour.GetActiveIter (out iter);
			
			store.AppendValues (new object [] {(string) Hour.Model.GetValue (iter, 0), Subject.Text});
			this.Window.Focus = Subject;
			
			
			int act = Hour.Active;
			Hour.RemoveText (act);
			Subject.Text = "";
			
			if ((Hour.Model.IterNChildren () - act) > 0)
				Hour.Active = act;
			else
				this.Window.Focus = (Gtk.Button) gui.GetWidget ("OkBtn");
			
			RemoveBtn.Sensitive = false;
			
			Subject.Completion.Model = CreateCompletion ();
		}
		
		
		public void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (LessonsList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store     = (ListStore) LessonsList.Model;
				Gtk.ListStore HourStore = (Gtk.ListStore) Hour.Model;
				
				Hour.AppendText ((string) store.GetValue (itr, 0));
				HourStore.SetSortColumnId (0, SortType.Ascending);
				
				store.Remove (ref itr);
				RemoveBtn.Sensitive = false;
			}
			
			Subject.Completion.Model = CreateCompletion ();
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			for (bool tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr))
				LessonsListStore.SetValue (itr, day, LessonsListGetSubject ((string) LessonsListStore.GetValue (itr, 0)));
			
			Window.Destroy ();
		}

		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public string LessonsListGetSubject (string time)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (time == (string) store.GetValue (itr, 0))
					return (string) store.GetValue (itr, 1);
			
			
			return "";
		}
		
		
		private Gtk.ListStore CreateCompletion ()
		{
			Gtk.ListStore mod = new Gtk.ListStore (typeof (string));
			ArrayList list    = GetLessonsList ();
			
			foreach (string item in list)
				mod.AppendValues (new object [] {item});
				
			return mod;
		}
		
		
		private ArrayList GetLessonsList ()
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = (Gtk.ListStore) LessonsList.Model;
			ArrayList     rtn   = new ArrayList ();
			
			for (bool tmp = LessonsListStore.GetIterFirst (out itr); tmp == true; tmp = LessonsListStore.IterNext (ref itr)) {
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 1));
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 2));
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 3));
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 4));
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 5));
				rtn = LessonsScheduleData.AddSubject (rtn, (string) LessonsListStore.GetValue (itr, 6));
			}
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				rtn = LessonsScheduleData.AddSubject (rtn, (string) store.GetValue (itr, 1));
			
			
			return rtn;
		}
	}
	
	
	/*
	 *  ExportLessonsScheduleWindow
	 */
	
	public class ExportingLessonsScheduleWindow {
		private Glade.XML     gui;

		[Glade.Widget] public  Gtk.Window     window;
		[Glade.Widget] private Gtk.Viewport   TitleViewport;
		[Glade.Widget] private Gtk.Image      ExportImg;
		[Glade.Widget] private Gtk.TreeView   SkinsList;
		[Glade.Widget] private Gtk.Entry      Title;
		[Glade.Widget] private Gtk.Entry      Directory;
		[Glade.Widget] private Gtk.VBox       ELSVBox;
		[Glade.Widget] private Gtk.VBox       ELSVBox2;
		[Glade.Widget] private Gtk.Button     OkBtn;
		

		
		public ExportingLessonsScheduleWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ExportingLessonsScheduleWindow", null);
			gui.Autoconnect (this);
			window = (Gtk.Window) gui.GetWidget ("ExportingLessonsScheduleWindow");
			
			TitleViewport.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			ExportImg.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "export-html.png");

			Directory.Text = FilesPaths.HomeDirectory ();
			
			
			ListStore store = new ListStore (typeof (string), typeof (string));
			SkinsList.AppendColumn (Catalog.GetString ("Name"), new CellRendererText (), "markup", 0);
			SkinsList.Model = store;

			
			foreach (Skin skn in GetSkinsList ())
			    store.AppendValues (new object [] {(string) skn.name, (string) skn.path});


			Gtk.TreeIter iter;
			if (store.GetIterFirst (out iter)) {
			    SkinsList.Selection.SelectIter (iter);
			    OkBtn.Sensitive = true;
			} else {
			    ELSVBox2.Sensitive = false;
			    Alert alert = new Alert (AlertType.Error,
					    Catalog.GetString ("No skins found !"),
					    Catalog.GetString ("You haven't installed any skins.\nClick cancel button to close this window."), false);
			    ELSVBox.Add (alert);
			}
		}


		public void on_SkinsList_cursor_changed (object o, EventArgs args)
		{
		    Gtk.TreeModel mod;
		    Gtk.TreeIter  itr;
		    
		    if (SkinsList.Selection.GetSelected (out mod, out itr))
			OkBtn.Sensitive = true;
		    else
			OkBtn.Sensitive = false;
		}


		public void on_BrowseBtn_clicked (object o, EventArgs args)
		{
		    Gtk.FileChooserDialog chooser = new Gtk.FileChooserDialog (Catalog.GetString ("Select folder"), window, Gtk.FileChooserAction.SelectFolder);
		    chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
		    chooser.AddButton (Stock.Ok, ResponseType.Ok);
		    chooser.DefaultResponse = ResponseType.Ok;

		    if (chooser.Run () == (int) ResponseType.Ok)
			Directory.Text = chooser.Filename;


		    chooser.Destroy ();
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
		    if (System.IO.Directory.Exists (Directory.Text) == false) {
			Alert alert = new Alert (AlertType.Error,
					    Catalog.GetString ("Directory doesn't exist !"),
					    Catalog.GetString ("Selected directory doesn't exists !\nClick <i>browse</i> button and select other directory !"));
			ELSVBox.Add (alert);

			return;
		    }


		    if (System.IO.File.Exists (Path.Combine (Directory.Text, "index.html")) == true) {
			Alert alert = new Alert (AlertType.Error,
					    Catalog.GetString ("File <i>index.html</i> exists !"),
					    Catalog.GetString ("File <i>index.html</i> exists in this directory !\nClick <i>browse</i> button and select other directory !"));
			ELSVBox.Add (alert);
			
			return;
		    }
		    
		    window.Sensitive = false;
		    
		    Gtk.TreeModel mod;
		    Gtk.TreeIter  itr;
		    SkinsList.Selection.GetSelected (out mod, out itr);
		    Gtk.ListStore store = (Gtk.ListStore) mod;
		    
		    
		    if (LessonsScheduleData.ExportToHTML ((string) store.GetValue (itr, 1), Directory.Text, Title.Text) == 1) {
			Alert alert = new Alert (AlertType.Error,
					    Catalog.GetString ("Exporting failed !"),
					    Catalog.GetString ("Selected skin is broken !\nSelect other skin."));
			ELSVBox.Add (alert);
			window.Sensitive = true;
		    } else {
			window.Destroy ();
		    }
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
		    window.Destroy ();
		}


		public void on_Title_activate (object o, EventArgs args)
		{
		    on_OkBtn_clicked (null, null);
		}


		public void on_Directory_activate (object o, EventArgs args)
		{
		    on_OkBtn_clicked (null, null);
		}
		
		
		private ArrayList GetSkinsList ()
		{
		    ArrayList rtn = new ArrayList ();

		    rtn = LessonsScheduleData.SkinsList (Path.Combine (
							    Path.Combine (FilesPaths.HomeDirectory (),
							    Path.Combine (FilesPaths.GnutuDirectory (), "skins")),
							    "lessonsschedule"), rtn);
		    
		    rtn = LessonsScheduleData.SkinsList (Path.Combine (
							    Path.Combine (System.AppDomain.CurrentDomain.BaseDirectory, "skins"),
							    "lessonsschedule"), rtn);
		    		    
		    return rtn;
		}
	}
}
