/*
 *  MainWindow.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using gnutu.GUI;
using gnutu.Widgets;
using gnutu;
using System.Runtime.InteropServices;
using System.Threading;


namespace gnutu.GUI {
	
	/*
	 * MainWindow class
	 */
	
	public class MainWindow {
		private Glade.XML gui;
		private DateTime  CurrentDate;
		private Entry     EntryCopy;
		private bool      IsTrayIcon;
		
		/* WIDGETS **********************************************************/
		[Glade.Widget] public  Gtk.Window   Window;
		[Glade.Widget] private Gtk.Calendar Calendar;
		[Glade.Widget] private Gtk.VBox     LeftBox;
		[Glade.Widget] private Gtk.Notebook Pages;
		[Glade.Widget] private Gtk.VBox     MsgVBox;
		private ButtonsSwitcher Buttons;
		public  GTrayIcon TrayIcon;
		
		[Glade.Widget] private Gtk.TreeView AtSchoolLessonsList;
		[Glade.Widget] private Gtk.TextView AtSchoolNotes;
		
		[Glade.Widget] private Gtk.TreeView AtHomeTasksList;
		
		[Glade.Widget] private Gtk.TextView DiaryTextView;
		
		[Glade.Widget] private Gtk.TreeView SearchList;
		private DateEdit SearchFromDate;
		private DateEdit SearchToDate;
		
		[Glade.Widget] private Gtk.TreeView StatsList;
		private DateEdit StatsFromDate;
		private DateEdit StatsToDate;
		
		private int CalendarMonth;
		
		
		/* WINDOW *************************************************************/
		public MainWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "MainWindow", null);
			gui.Autoconnect (this);
			this.Window = (Gtk.Window) gui.GetWidget ("MainWindow");
			this.Window.Show ();
			this.Window.Icon = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "logo-20.png");
			
			this.Window.Sensitive = false;
			
			while (Application.EventsPending ())
	        		Application.RunIteration ();
            
			// Open and load settings
			Settings.Open ();
			
			// Pages - hide tabs
			Pages.ShowTabs = false;
			
			// Toolbar
			Gtk.CheckMenuItem ShowToolbar = (Gtk.CheckMenuItem) gui.GetWidget ("ShowToolbar");
			Gtk.Toolbar       MainToolbar = (Gtk.Toolbar)       gui.GetWidget ("MainToolbar");
			
			MainToolbar.Visible = Settings.GetBool ("ShowToolbar");
			ShowToolbar.Active  = Settings.GetBool ("ShowToolbar");
			
			// buttons switcher
			Buttons = new ButtonsSwitcher (Pages, false);
			LeftBox.PackEnd (Buttons, false, true, 3);
			Buttons.Show ();
			
			Buttons.Add (Catalog.GetString ("Summary"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Day-20.png")));
			Buttons.Add (Catalog.GetString ("At school"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtSchool-20.png")));
			Buttons.Add (Catalog.GetString ("At home"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtHome-20.png")));
			Buttons.Add (Catalog.GetString ("Diary"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Diary-20.png")));
			Buttons.AddSeparator ();
			Buttons.Add (Catalog.GetString ("Search"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Search-20.png")));
			Buttons.Add (Catalog.GetString ("Stats"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Stats-20.png")));
			
			// set menu icons
			Gtk.ImageMenuItem lessons_schedule = (Gtk.ImageMenuItem) gui.GetWidget ("lessons_schedule");
			lessons_schedule.Image = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "LessonsSchedule-16.png"));
			
			Gtk.ImageMenuItem search_tests = (Gtk.ImageMenuItem) gui.GetWidget ("search_tests");
			search_tests.Image = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "SearchTests-16.png"));
			
			Gtk.ImageMenuItem days_off_school = (Gtk.ImageMenuItem) gui.GetWidget ("days_off_school");
			days_off_school.Image = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "DaysOffSchool-16.png"));
			
			// set buttons icons
			Gtk.ToolButton BackBtn = (Gtk.ToolButton) gui.GetWidget ("BackBtn");
			BackBtn.IconWidget = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Previous-20.png"));
			BackBtn.IconWidget.Show ();
			
			Gtk.ToolButton TodayBtn = (Gtk.ToolButton) gui.GetWidget ("TodayBtn");
			TodayBtn.IconWidget = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Home-20.png"));
			TodayBtn.IconWidget.Show ();
			
			Gtk.ToolButton NextBtn = (Gtk.ToolButton) gui.GetWidget ("NextBtn");
			NextBtn.IconWidget = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Next-20.png"));
			NextBtn.IconWidget.Show ();
			
			// Open and load Database
			DaysOffSchool.Open ();
			LessonsScheduleData.Open ();
			DB.Open ();
			
			// tabs
			AtSchoolCreate ();
			AtHomeCreate ();
			DiaryCreate ();
			SearchCreate ();
			StatsCreate ();
			Buttons.GoToPage (0);
			
			
			LoadEntry (this.Calendar.Date, false);
			SummaryCreate ();
			TrayIconCreate ();
			
			
			GLib.Timeout.Add (1000, AtHomeReminder);
			
			CalendarMonth = this.Calendar.Month;
			if (Settings.GetBool ("MarkDaysOffSchool") == true)
				CalendarMarkDayOffSchool ();
			
			
			// Checking updates
			if (Settings.GetBool ("InformationsAboutUpdates") == true) {
				CheckUpdates.MsgVBox = this.MsgVBox;
				ThreadStart run = new ThreadStart (CheckUpdates.Check);
				System.Threading.Thread thread = new System.Threading.Thread (run);
				thread.Start ();
				//GLib.Timeout.Add (1000, CheckUpdates.Check);
			}

			
			this.Window.Sensitive = true;
			while (Application.EventsPending ())
        		    	Application.RunIteration ();
		}


		public void on_MainWindow_delete_event (object o, DeleteEventArgs args) 
		{
			if ((Settings.GetBool ("MinimalizeWindowToTray") == false) || (IsTrayIcon == false)) {
				on_exit_activate (null, null);
				args.RetVal = true;
			} else {
				if (Settings.GetBool ("CloseHideMsg") == true) {
					TrayMessage msg = new TrayMessage (MsgType.Info, true, null, null,
										Catalog.GetString ("GNUTU window has hidden"),
										Catalog.GetString ("To restore window click GNUTU icon in your notification area."));
					msg.Show ();
					Settings.SetBool ("CloseHideMsg", false);
				}
				
				this.Window.Hide ();
				args.RetVal = true;
			}
		}
		
		
		public void on_Pages_switch_page (object o, SwitchPageArgs args)
		{
			SummaryCreate ();
			
			if ((Pages.Page == 4) || (Pages.Page == 5))
				LoadEntry (CurrentDate, true);
		}
		
		
		public void on_exit_activate (object o, EventArgs args)
		{
			this.Window.Sensitive = false;
			while (Application.EventsPending ())
			    Application.RunIteration ();
            
            
			// save database
			LoadEntry (this.Calendar.Date, true);
			DB.Close ();

			// save settings
			Settings.Close ();
			
			// save lessons schedule
			LessonsScheduleData.Close ();
			
			// save DaysOffSchool
			DaysOffSchool.Close ();


			this.Window.Sensitive = false;
			while (Application.EventsPending ())
			    Application.RunIteration ();
            
			TrayIcon.Hide ();
			Application.Quit ();
		}
		

		public void on_search_tests_activate (object o, EventArgs args)
		{
			SearchingTestsWindow wdw = new SearchingTestsWindow ();
			wdw.Window.Show ();
		}
		
		
		public void on_insert_subjects_activate (object o, EventArgs args)
		{
			InsertSubjects ();
		}
		
		
		public void on_InsertSubjectsMonday_activate (object o, EventArgs args)
		{
			InsertSubjects (1);
		}
		
		
		public void on_InsertSubjectsTuesday_activate (object o, EventArgs args)
		{
			InsertSubjects (2);
		}
		
		
		public void on_InsertSubjectsWednesday_activate (object o, EventArgs args)
		{
			InsertSubjects (3);
		}
		
		
		public void on_InsertSubjectsThursday_activate (object o, EventArgs args)
		{
			InsertSubjects (4);
		}
		
		
		public void on_InsertSubjectsFriday_activate (object o, EventArgs args)
		{
			InsertSubjects (5);
		}
		
		
		public void on_InsertSubjectsSaturday_activate (object o, EventArgs args)
		{
			InsertSubjects (6);
		}
		
		
		public void on_ShowToolBar_activate (object o, EventArgs args)
		{
			Gtk.CheckMenuItem ShowToolbar = (Gtk.CheckMenuItem) gui.GetWidget ("ShowToolbar");
			Gtk.Toolbar       MainToolbar = (Gtk.Toolbar)       gui.GetWidget ("MainToolbar");
			
			if (ShowToolbar.Active == true)
				MainToolbar.Visible = true;
			else
				MainToolbar.Visible = false;
			
			Settings.SetBool ("ShowToolbar", ShowToolbar.Active);
		}
		
		
		public void on_lessons_schedule_activate (object o, EventArgs args)
		{
			LessonsScheduleWindow wdw = new LessonsScheduleWindow ();
			wdw.Window.Show ();
		}
		
		
		public void on_settings_activate (object o, EventArgs args)
		{
			EntryCopy = GetEntry ();
			
			SettingsWindow wdw = new SettingsWindow ();
			wdw.Window.Destroyed += new EventHandler (on_settings_window_destroy);
			wdw.Window.Show ();
		}
		
		
		public void on_settings_window_destroy (object o, EventArgs args)
		{
			SetEntry (EntryCopy);
			SummaryCreate ();
			
			if (Settings.GetBool ("MarkDaysOffSchool") == true)
			    CalendarMarkDayOffSchool ();
			else
			    this.Calendar.ClearMarks ();
		}


		public void on_days_off_school_activate (object o, EventArgs args)
		{
			DaysOffSchoolWindow wdw = new DaysOffSchoolWindow ();
			wdw.Window.Destroyed += new EventHandler (on_days_off_school_window_destroy);
			wdw.Window.Show ();
		}


		public void on_days_off_school_window_destroy (object o, EventArgs args)
		{
			if (Settings.GetBool ("MarkDaysOffSchool") == true)
			    CalendarMarkDayOffSchool ();
		}

		

		public void on_about_activate (object o, EventArgs args)
		{
			AboutWindow wdw = new AboutWindow ();
			wdw.Window.Show ();
		}
			
		
		public void on_BackBtn_clicked (object o, EventArgs args)
		{
			this.Calendar.Date = this.Calendar.Date.AddDays (-1);
		}


		public void on_TodayBtn_clicked (object o, EventArgs args)
		{
			this.Calendar.Date = System.DateTime.Today;
		}


		public void on_NextBtn_clicked (object o, EventArgs args)
		{
			this.Calendar.Date = this.Calendar.Date.AddDays (1);
		}
		
		
		public void on_ExitBtn_clicked (object o, EventArgs args)
		{
			on_exit_activate (null, null);
		}
		
		
		public void on_Calendar_day_selected (object o, EventArgs args)
		{
			LoadEntry (this.Calendar.Date, true);
		}
		
		
		public void on_Calendar_month_changed (object o, EventArgs args)
		{
			if (this.Calendar.Month != CalendarMonth) {
				CalendarMonth = this.Calendar.Month;
				
				if (Settings.GetBool ("MarkDaysOffSchool") == true)
					CalendarMarkDayOffSchool ();
				else
					this.Calendar.ClearMarks ();
			}
		}
		
		
		public void on_Calendar_prev_year (object o, EventArgs args)
		{
			if (this.Calendar.Year == 1969)
				this.Calendar.Year = 1970;
			
			if (Settings.GetBool ("MarkDaysOffSchool") == true)
				CalendarMarkDayOffSchool ();
			else
				this.Calendar.ClearMarks ();
		}
		
		
		public void on_Calendar_next_year (object o, EventArgs args)
		{
			if (Settings.GetBool ("MarkDaysOffSchool") == true)
				CalendarMarkDayOffSchool ();
			else
				this.Calendar.ClearMarks ();
		}
		
		
		private void CalendarMarkDayOffSchool ()
		{
			this.Calendar.ClearMarks ();
			
			foreach (DayOffSchool day in DaysOffSchool.data)
				if ((day.date.month == (this.CalendarMonth + 1)) && (day.date.year == this.Calendar.Year))
					this.Calendar.MarkDay ((uint) day.date.day);
		}
		
		
		/* TrayIcon ***********************************************************/
		public void TrayIconCreate ()
		{
			TrayIcon = new GTrayIcon ("GNUTU", this.Window, on_ExitBtn_clicked);
			this.IsTrayIcon = TrayIcon.IsTrayIcon;
		}
		
		
		/* Summary ************************************************************/
		public void SummaryCreate ()
		{
			Gtk.Image SummaryIcon  = (Gtk.Image) gui.GetWidget ("SummaryIcon");
			SummaryIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Day-20.png");
			
			
			SummarySetToday ();
			SummarySetDayOffSchool ();
			
			if (Settings.GetBool ("ShowTestAndQuizes"))
				SummarySetTests ();
			
			if (Settings.GetBool ("ShowNotes"))
				SummarySetNotes ();
			
			if (Settings.GetBool ("ShowToDoList"))
				SummarySetToDo ();
			
			if (Settings.GetBool ("ShowDiary"))
				SummarySetDiary ();
			
			if (Settings.GetBool ("ShowLessonsList"))
				SummarySetLessons ();
			
			if (Settings.GetBool ("ShowDaysLeft"))
				SummarySetDaysLeft ();
			
			if (Settings.GetBool ("ShowNearestTests"))
				SummarySetNearestTests ();
		}
		
		
		public void SummarySetToday ()
		{
			Gtk.Label lbl = (Gtk.Label) gui.GetWidget ("SummaryTodayLabel");
			lbl.Markup = "<span weight=\"bold\" size=\"larger\">" + Catalog.GetString ("Today's date is ") + new Date (CurrentDate).ToString (Settings.GetBool ("DateFirstYear")) + "</span>";
		}
		
		
		public void SummarySetDayOffSchool ()
		{
			Gtk.Label SummaryAboutDay = (Gtk.Label) gui.GetWidget ("SummaryAboutDay");
			
			if (DaysOffSchool.IsDayOffSchool (new Date (CurrentDate)) == true) {
				string desc = DaysOffSchool.GetDayDescription (new Date (CurrentDate));
				if (desc.Length > 0)
					SummaryAboutDay.Markup = Catalog.GetString ("Day off school:") + " " + DaysOffSchool.GetDayDescription (new Date (CurrentDate));
				else
					SummaryAboutDay.Markup = Catalog.GetString ("Day off school");
				
				SummaryAboutDay.Visible = true;
			} else
				SummaryAboutDay.Visible = false;
		}
		
		
		public void SummarySetTests ()
		{
			Gtk.Frame SummaryTestsFrame = (Gtk.Frame) gui.GetWidget ("SummaryTestsFrame");
			Gtk.Label SummaryTestsText  = (Gtk.Label) gui.GetWidget ("SummaryTestsText");
			
			Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
			Gtk.TreeIter  iter;
			
			string txt = "";
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				if ((string) store.GetValue (iter, 3) != "") {
					string test = (string) store.GetValue (iter, 3);
					test = test.Replace ("<span color='#ec9000'>", "");
					test = test.Replace ("<span color='#FF0000'>", "");
					test = test.Replace ("</span>", "");
					
					txt += (string) store.GetValue (iter, 1) + " - <i>" + test + "</i>\n";
				}
			}
			
			if (txt.Length > 0) {
				SummaryTestsText.Markup = txt;
				SummaryTestsFrame.Visible = true;
			} else
				SummaryTestsFrame.Visible = false;
		}
		
		
		public void SummarySetNotes ()
		{
			Gtk.Frame SummaryNotesFrame = (Gtk.Frame) gui.GetWidget ("SummaryNotesFrame");
			Gtk.Label SummaryNotesText  = (Gtk.Label) gui.GetWidget ("SummaryNotesText");
			
			if (AtSchoolNotes.Buffer.Text.Length > 0) {
				string txt = "";
				
				if (AtSchoolNotes.Buffer.Text.Length > 100)
					txt = AtSchoolNotes.Buffer.Text.Substring (0, 100) + "...";
				else
					txt = AtSchoolNotes.Buffer.Text;
				
				SummaryNotesText.Markup = txt.Replace ('\n', ' ');
				SummaryNotesFrame.Visible = true;
			} else
				SummaryNotesFrame.Visible = false;
		}
		
		
		public void SummarySetToDo ()
		{
			Gtk.Frame SummaryToDoFrame = (Gtk.Frame) gui.GetWidget ("SummaryToDoFrame");
			Gtk.Label SummaryToDoText  = (Gtk.Label) gui.GetWidget ("SummaryToDoText");
			
			Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
			Gtk.TreeIter  iter;
			
			bool   ShowCompletedTasks = Settings.GetBool ("ShowCompletedTasks");
			string txt = "";
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				if (((bool) store.GetValue (iter, 0) == false) || (ShowCompletedTasks == true)) {
					if ((bool) store.GetValue (iter, 0) == true)
						txt += "<b>-</b> " + Catalog.GetString ("[COMPLETED]") + " " + (string) store.GetValue (iter, 2) + "\n";
					else
						txt += "<b>-</b> " + (string) store.GetValue (iter, 2) + "\n";
				}
			}
			
			if (txt.Length > 0) {
				SummaryToDoText.Markup = txt;
				SummaryToDoFrame.Visible = true;
			} else
				SummaryToDoFrame.Visible = false;
		}
		
		
		public void SummarySetDiary ()
		{
			Gtk.Frame SummaryDiaryFrame = (Gtk.Frame) gui.GetWidget ("SummaryDiaryFrame");
			Gtk.Label SummaryDiaryText  = (Gtk.Label) gui.GetWidget ("SummaryDiaryText");
			
			if (DiaryTextView.Buffer.Text.Length > 0) {
				string txt = "";
				
				if (DiaryTextView.Buffer.Text.Length > 100)
					txt = DiaryTextView.Buffer.Text.Substring (0, 100) + "...";
				else
					txt = DiaryTextView.Buffer.Text;
				
				SummaryDiaryText.Markup = txt.Replace ('\n', ' ');
				SummaryDiaryFrame.Visible = true;
			} else
				SummaryDiaryFrame.Visible = false;
		}
		
		
		public void SummarySetLessons ()
		{
			Gtk.Frame SummaryLessonsFrame = (Gtk.Frame) gui.GetWidget ("SummaryLessonsFrame");
			Gtk.Label SummaryLessonsText  = (Gtk.Label) gui.GetWidget ("SummaryLessonsText");
			
			Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
			Gtk.TreeIter  iter;
			
			string txt = "";
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter))
				txt += (string) store.GetValue (iter, 0) + " - " + (string) store.GetValue (iter, 1) + "\n";
			
			if (txt.Length > 0) {
				SummaryLessonsText.Markup = txt;
				SummaryLessonsFrame.Visible = true;
			} else
				SummaryLessonsFrame.Visible = false;
		}
		
		
		public void SummarySetDaysLeft ()
		{
			Gtk.Frame SummaryDaysLeftFrame = (Gtk.Frame) gui.GetWidget ("SummaryDaysLeftFrame");
			Gtk.Label SummaryDaysLeftHead  = (Gtk.Label) gui.GetWidget ("SummaryDaysLeftHead");
			Gtk.Label SummaryDaysLeftText  = (Gtk.Label) gui.GetWidget ("SummaryDaysLeftText");
			
			DateTime SchoolYearBegin = new Date (Settings.Get ("SchoolYearBegin"), false).ToDateTime ();
			DateTime SchoolYearEnd   = new Date (Settings.Get ("SchoolYearEnd"), false).ToDateTime ();
			DateTime HolidayEnd      = new Date (Settings.Get ("HolidayEnd"), false).ToDateTime ();
			
			
			if ((CurrentDate >= SchoolYearBegin) && (CurrentDate <= SchoolYearEnd)) {
				TimeSpan r = SchoolYearEnd.Subtract (CurrentDate);
				SummaryDaysLeftHead.Markup = Catalog.GetString ("<b>Days left until the end of the school year</b>");
				SummaryDaysLeftText.Markup = r.Days.ToString ();
				SummaryDaysLeftFrame.Visible = true;
			} else if ((CurrentDate > SchoolYearEnd) && (CurrentDate <= HolidayEnd)) {
				TimeSpan r = HolidayEnd.Subtract (CurrentDate);
				SummaryDaysLeftHead.Markup = Catalog.GetString ("<b>Days left until the end of the holiday</b>");
				SummaryDaysLeftText.Markup = r.Days.ToString ();
				SummaryDaysLeftFrame.Visible = true;
			} else
				SummaryDaysLeftFrame.Visible = false;
		}
		
		
		public void SummarySetNearestTests ()
		{
			Gtk.Frame SummaryNearestTestsFrame = (Gtk.Frame) gui.GetWidget ("SummaryNearestTestsFrame");
			Gtk.Label SummaryNearestTestsText  = (Gtk.Label) gui.GetWidget ("SummaryNearestTestsText");
			
			ArrayList tests = DB.SearchTests (CurrentDate.AddDays (1), CurrentDate.AddDays (14), true, true);
			
			string text = "";
			foreach (SearchTestsResult res in tests) {
				text += "<b>" + res.date.ToString (Settings.GetBool ("DateFirstYear"));
				text += " "   + res.mod;
				text += " - " + res.name + "</b>\n";
				text += "<i>" + res.desc + "</i>\n\n";
			}
			
			if (text.Length > 0) {
				SummaryNearestTestsText.Markup = text;
				SummaryNearestTestsFrame.Visible = true;
			} else
				SummaryNearestTestsFrame.Visible = false;
		}
		
		
		public void on_SummaryNotesMoreBtn_clicked (object o, EventArgs args)
		{
			this.Buttons.GoToPage (1);
		}
		
		
		public void on_SummaryDiaryMoreBtn_clicked (object o, EventArgs args)
		{
			this.Buttons.GoToPage (3);
		}
		
		
		
		/* AtSchool ***********************************************************/
		public void AtSchoolCreate ()
		{
			Gtk.Image AtSchoolIcon = (Gtk.Image) gui.GetWidget ("AtSchoolIcon");
			AtSchoolIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtSchool-20.png");
			
			
			ListStore store = new ListStore (typeof (string), typeof (string),  typeof (string), typeof (string));
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			AtSchoolLessonsList.Model = store;
			
			AtSchoolLessonsList.AppendColumn (Catalog.GetString ("Time"), new CellRendererText (), "text", 0);
			AtSchoolLessonsList.AppendColumn (Catalog.GetString ("Subjects"), new CellRendererText (), "text", 1);
			AtSchoolLessonsList.AppendColumn (Catalog.GetString ("Marks"), new CellRendererText (), "text", 2);
			AtSchoolLessonsList.AppendColumn (Catalog.GetString ("Tests"), new CellRendererText (), "markup", 3);
			
			store.SetSortFunc (0, new Gtk.TreeIterCompareFunc (AtSchoolLessonsListCompare));
		}
		
		
		public void on_AtSchoolAddSubject_clicked (object o, EventArgs args)
		{
			EditSubjectWindow wdw = new EditSubjectWindow (AtSchoolLessonsList.Model);
			wdw.Window.Show ();
		}
		
		
		public void on_AtSchoolRemoveSubject_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtSchoolLessonsList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) mod;
				store.Remove (ref itr);
				
				// Changing buttons sensitive
				Gtk.Button RemoveSubject = (Gtk.Button) gui.GetWidget ("AtSchoolRemoveSubject");
				Gtk.Button ModifySubject = (Gtk.Button) gui.GetWidget ("AtSchoolModifySubject");
				
				RemoveSubject.Sensitive = false;
				ModifySubject.Sensitive = false;
			}
		}
		
		
		public void on_AtSchoolModifySubject_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtSchoolLessonsList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) mod;
			
				EditSubjectWindow wdw = new EditSubjectWindow (AtSchoolLessonsList.Model,
										(string) store.GetValue (itr, 0),
										(string) store.GetValue (itr, 1),
										(string) store.GetValue (itr, 2),
										(string) store.GetValue (itr, 3));
				wdw.Window.Show ();
				
				// Changing buttons sensitive
				Gtk.Button RemoveSubject = (Gtk.Button) gui.GetWidget ("AtSchoolRemoveSubject");
				Gtk.Button ModifySubject = (Gtk.Button) gui.GetWidget ("AtSchoolModifySubject");
				
				RemoveSubject.Sensitive = false;
				ModifySubject.Sensitive = false;
			}
		}
		
		
		public void on_AtSchoolLessonsList_cursor_changed (object o, EventArgs args)
		{
			Gtk.Button RemoveTask = (Gtk.Button) gui.GetWidget ("AtSchoolRemoveSubject");
			Gtk.Button ModifyTask = (Gtk.Button) gui.GetWidget ("AtSchoolModifySubject");
			
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtSchoolLessonsList.Selection.GetSelected (out mod, out itr)) {
				RemoveTask.Sensitive = true;
				ModifyTask.Sensitive = true;
			} else {
				RemoveTask.Sensitive = false;
				ModifyTask.Sensitive = false;
			}
		}
		
		
		public void on_AtSchoolLessonsList_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.Button == 3) && (AtSchoolLessonsList.Selection.CountSelectedRows () > 0)) {
			    Glade.XML m = Glade.XML.FromAssembly ("gui.glade", "LessonsListMenu", null);
			    m.Autoconnect (this);
			    Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("LessonsListMenu");

			    Gtk.ImageMenuItem ic = (Gtk.ImageMenuItem) m.GetWidget ("add_homework_to_todo_list");
			    ic.Image = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "add-homework-16.png"));

			    menu.Popup ();
			}
		}
		
		
		public void on_AtSchoolLessonsList_row_activated (object o, RowActivatedArgs args)
		{
			on_AtSchoolModifySubject_clicked (null, null);
		}
		
		
		private int AtSchoolLessonsListCompare (Gtk.TreeModel model, Gtk.TreeIter a, Gtk.TreeIter b)
		{
			Time t1 = new Time ((string) model.GetValue (a, 0), Settings.GetBool ("TimeMode24"));
			Time t2 = new Time ((string) model.GetValue (b, 0), Settings.GetBool ("TimeMode24"));
			
			if (t1.ToDateTime () > t2.ToDateTime ())
				return 1;
			else if (t1.ToDateTime () == t2.ToDateTime ())
				return 0;
			
			return -1;
		}
		
		
		private void InsertSubjects ()
		{
			InsertSubjects (this.Calendar.Date.DayOfWeek.value__);
		}
		
		
		private void InsertSubjects (int DayID)
		{
			if (DayID == 0)
				return;
			
			
			ClearEntry (false, true, false, false);
			Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
			
			ArrayList time = LessonsScheduleData.Get (0);
			ArrayList day  = LessonsScheduleData.Get (DayID);
			
			int i = 0;
			while (i < time.Count) {
				if ((string) day [i] != "")
					store.AppendValues (new object [] {(string) time [i], (string) day [i], "", ""});
				
				i++;
			}
		}


		public void on_add_homework_to_todo_list_activate (object o, EventArgs args)
		{
		    Gtk.TreeModel mod;
		    Gtk.TreeIter  itr;
		    
		    if (AtSchoolLessonsList.Selection.GetSelected (out mod, out itr)) {
			Gtk.ListStore store = (Gtk.ListStore) mod;

			Gtk.ListStore ToDoStore = (Gtk.ListStore) AtHomeTasksList.Model;
			ToDoStore.AppendValues (new object [] {false,
							    new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-3.png")),
							    Catalog.GetString ("Homework: ") + (string) store.GetValue (itr, 1), "3",
							    false, (string) "00:00", 0});
		    }
		}
		
		
		public void on_remove_subject_activate (object o, EventArgs args)
		{
		    on_AtSchoolRemoveSubject_clicked (null, null);
		}
		
		
		public void on_modify_subject_activate (object o, EventArgs args)
		{
		    on_AtSchoolModifySubject_clicked (null, null);
		}
		
		
		public void on_remove_all_subjects_activate (object o, EventArgs args)
		{
		    Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
		    store.Clear ();
		}
		
		
		
		/* AtHome *************************************************************/
		public void AtHomeCreate ()
		{
			Gtk.Image AtHomeIcon   = (Gtk.Image) gui.GetWidget ("AtHomeIcon");
			AtHomeIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtHome-20.png");
			
			
			ListStore TasksStore = new ListStore (typeof (bool), typeof (Gdk.Pixbuf), typeof (string), typeof (string),
												  typeof (bool), typeof (string), typeof (int));
												  
			AtHomeTasksList.Model = TasksStore;
			
			CellRendererToggle tgl = new CellRendererToggle ();
			tgl.Toggled += on_completed_toggled;
			AtHomeTasksList.AppendColumn (Catalog.GetString ("Completed"), tgl, "active", 0);
			AtHomeTasksList.AppendColumn (Catalog.GetString ("Priority"), new CellRendererPixbuf (), "pixbuf", 1);
			AtHomeTasksList.AppendColumn (Catalog.GetString ("Task"), new CellRendererText (), "text", 2);
			
			TasksStore.SetSortColumnId (3, SortType.Descending);
		}


		public void on_AtHomeTasksList_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.Button == 3) && (AtHomeTasksList.Selection.CountSelectedRows () > 0)) {
			    Glade.XML m = Glade.XML.FromAssembly ("gui.glade", "TasksListMenu", null);
			    m.Autoconnect (this);
			    Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("TasksListMenu");

			    menu.Popup ();
			}
		}


		public void on_completed_toggled (object o, ToggledArgs args)
		{
		    Gtk.TreeIter iter;
		    Gtk.ListStore TasksStore = (Gtk.ListStore) AtHomeTasksList.Model;
		    
		    if (TasksStore.GetIter (out iter, new Gtk.TreePath (args.Path))) {
			bool val = (bool) TasksStore.GetValue(iter, 0);
			TasksStore.SetValue (iter, 0, !val);
		    }
		}

		
		public void on_AtHomeAddTask_clicked (object o, EventArgs args)
		{
			ModifyTaskWindow wdw = new ModifyTaskWindow ((Gtk.ListStore) AtHomeTasksList.Model);
			wdw.Window.Show ();
		}
		
		
		public void on_AtHomeModifyTask_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtHomeTasksList.Selection.GetSelected (out mod, out itr)) {
				ModifyTaskWindow wdw = new ModifyTaskWindow ((Gtk.ListStore) mod, itr);
				wdw.Window.Show ();
			}
		}
		
		
		public void on_AtHomeRemoveTask_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtHomeTasksList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
				store.Remove (ref itr);
				
				Gtk.Button RemoveTask      = (Gtk.Button)       gui.GetWidget ("AtHomeRemoveTask");
				Gtk.Button ModifyTask      = (Gtk.Button)       gui.GetWidget ("AtHomeModifyTask");
				Gtk.ToggleButton Completed = (Gtk.ToggleButton) gui.GetWidget ("AtHomeCompletedBtn");
				
				RemoveTask.Sensitive = false;
				ModifyTask.Sensitive = false;
				Completed.Sensitive  = false;
			}
		}
		
		
		public void on_AtHomeCompletedBtn_toggled (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtHomeTasksList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ToggleButton btn = (Gtk.ToggleButton) o;
				
				Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
				store.SetValue (itr, 0, btn.Active);
			}
		}
		
		
		public void on_AtHomeTasksList_cursor_changed (object o, EventArgs args)
		{
			Gtk.Button RemoveSubject   = (Gtk.Button)       gui.GetWidget ("AtHomeRemoveTask");
			Gtk.Button ModifySubject   = (Gtk.Button)       gui.GetWidget ("AtHomeModifyTask");
			Gtk.ToggleButton Completed = (Gtk.ToggleButton) gui.GetWidget ("AtHomeCompletedBtn");
			
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (AtHomeTasksList.Selection.GetSelected (out mod, out itr)) {
				RemoveSubject.Sensitive = true;
				ModifySubject.Sensitive = true;
				Completed.Sensitive     = true;
				
				Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
				Completed.Active    = (bool) store.GetValue (itr, 0);
			} else {
				RemoveSubject.Sensitive = false;
				ModifySubject.Sensitive = false;
				Completed.Sensitive     = false;
			}
		}
		
		
		public void on_AtHomeTasksList_row_activated (object o, RowActivatedArgs args)
		{
			on_AtHomeModifyTask_clicked (null, null);
		}
		
		
		public bool AtHomeReminder ()
		{
			if (DateTime.Now.Second != 0)
				return true;
			
			if (CurrentDate == DateTime.Today) {
				Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
				Gtk.TreeIter  iter;
				
				for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
					TimeSpan ts = new Time (DateTime.Now).ToDateTime () - new Time ((string) store.GetValue (iter, 5), true).ToDateTime ();
					
					if (((bool) store.GetValue (iter, 4) == true) &&
						((bool) store.GetValue (iter, 0) == false) &&
						(ts.Minutes >= 0) && (ts.Hours >= 0) &&
						((int) store.GetValue (iter, 6) > 0) && 
						((ts.Minutes % (int) store.GetValue (iter, 6)) == 0)) {
						
						TrayMessage msg = new TrayMessage ((string) store.GetValue (iter, 2), (Gdk.Pixbuf) store.GetValue (iter, 1), store, iter);
						msg.Show ();
					} else if (((bool) store.GetValue (iter, 4) == true) &&
						((bool) store.GetValue (iter, 0) == false) &&
						(ts.Minutes >= 0) && (ts.Hours >= 0) &&
						((int) store.GetValue (iter, 6) == 0)) {
						
						TrayMessage msg = new TrayMessage ((string) store.GetValue (iter, 2), (Gdk.Pixbuf) store.GetValue (iter, 1), store, iter);
						msg.Show ();
					}
				}
			} else {
				gnutu.Entry ent = DB.GetEntryByDate (DateTime.Today);
				int         i   = 0;

				if (ent.todo == null)
					return true;
		
				foreach (ToDo task in ent.todo) {
					TimeSpan ts = DateTime.Now - task.ReminderAt.ToDateTime ();
					
					if ((task.Reminder == true) &&
						(task.did == false) &&
						(ts.Minutes >= 0) && (ts.Hours >= 0) &&
						(task.ReminderCount > 0) && 
						((ts.Minutes % task.ReminderCount) == 0)) {
						
						TrayMessage msg = new TrayMessage (task.item, (int) task.priority, i, CurrentDate);
						msg.Show ();
					} else if ((task.Reminder == true) &&
						(task.did == false) &&
						(ts.Minutes >= 0) && (ts.Hours >= 0) &&
						(task.ReminderCount == 0)) {
						
						TrayMessage msg = new TrayMessage (task.item, (int) task.priority, i, CurrentDate);
						msg.Show ();
					}
					
					i++;
				}
			}
			
			return true;
		}


		public void on_add_a_task_activate (object o, EventArgs args)
		{
		    on_AtHomeAddTask_clicked (null, null);
		}


		public void on_remove_task_activate (object o, EventArgs args)
		{
		    on_AtHomeRemoveTask_clicked (null, null);
		}
		

		public void on_modify_task_activate (object o, EventArgs args)
		{
		    on_AtHomeModifyTask_clicked (null, null);
		}
		
		
		
		/* Diary **************************************************************/
		public void DiaryCreate ()
		{		
			Gtk.Image DiaryIcon    = (Gtk.Image) gui.GetWidget ("DiaryIcon");
			DiaryIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Diary-20.png");
		}
		
		
		/* Search *************************************************************/
		public void SearchCreate ()
		{
			Gtk.Image SearchIcon   = (Gtk.Image) gui.GetWidget ("SearchIcon");
			SearchIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Search-20.png");
			
			Gtk.HBox SearchFromHBox = (Gtk.HBox) gui.GetWidget ("SearchFromHBox");
			Gtk.HBox SearchToHBox   = (Gtk.HBox) gui.GetWidget ("SearchToHBox");
			
			Gtk.Image SearchBtnIcon = (Gtk.Image) gui.GetWidget ("SearchBtnIcon");
			SearchBtnIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Search-20.png");
			
			SearchFromDate = new DateEdit ();
			SearchToDate   = new DateEdit ();
			StatsFromDate  = new DateEdit ();
			StatsToDate    = new DateEdit ();
			
			StatsFromDate.SetDate (new Date (Settings.Get ("SchoolYearBegin"), false).ToDateTime ());
			
			SearchFromHBox.Add (SearchFromDate);
			SearchToHBox.Add (SearchToDate);
			
			
			ListStore store2 = new ListStore (typeof (string), typeof (Gdk.Pixbuf), typeof (string));
			SearchList.Model = store2;
			
			SearchList.AppendColumn (Catalog.GetString ("Date"), new CellRendererText (), "markup", 0);
			SearchList.AppendColumn (Catalog.GetString ("Score"), new CellRendererPixbuf (), "pixbuf", 1);
			SearchList.AppendColumn (Catalog.GetString ("Category"), new CellRendererText (), "markup", 2);
		}
		
		
		public void on_SearchBtn_clicked (object o, EventArgs args)
		{
			Gtk.Button    SearchShowBtn = (Gtk.Button) gui.GetWidget ("SearchShowBtn");
			Gtk.Entry     query         = (Gtk.Entry)  gui.GetWidget ("SearchQuery");
			Gtk.ListStore store         = (ListStore)  SearchList.Model;
			
			Gtk.CheckButton SearchNotes      = (Gtk.CheckButton) gui.GetWidget ("SearchNotes");
			Gtk.CheckButton SearchTests      = (Gtk.CheckButton) gui.GetWidget ("SearchTests");
			Gtk.CheckButton SearchTasks      = (Gtk.CheckButton) gui.GetWidget ("SearchTasks");
			Gtk.CheckButton SearchDiary      = (Gtk.CheckButton) gui.GetWidget ("SearchDiary");
			Gtk.CheckButton SearchAllEntries = (Gtk.CheckButton) gui.GetWidget ("SearchAllEntries");
			
			
			store.Clear ();
			SearchShowBtn.Sensitive = false;
			
			ArrayList arr = DB.Search (query.Text,
							SearchNotes.Active, SearchTests.Active, SearchTasks.Active, SearchDiary.Active,
							SearchFromDate.GetDate (), SearchToDate.GetDate (), SearchAllEntries.Active);
			
			if (arr.Count == 0) {
			    store.AppendValues (new object [] {"", null, Catalog.GetString ("No results found !")});
			    return;
			}
			
			foreach (SearchResult res in arr)
				store.AppendValues (new object [] {res.date.ToString (Settings.GetBool ("DateFirstYear")),
											new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Stars-" + res.score.ToString () + ".png")),
											res.CategoryToString ()});
		}


		public void on_SearchShowBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeView  SearchLists = (Gtk.TreeView) gui.GetWidget ("SearchList");
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SearchLists.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) mod;
				Date date = new Date ((string) store.GetValue (itr, 0), Settings.GetBool ("DateFirstYear"));
				
				this.Calendar.Date = date.ToDateTime ();
				SearchResult res = (SearchResult) DB.SearchArray [SearchList.Selection.GetSelectedRows () [0].Indices [0]];
				
				if (res.category == SearchCategory.tasks)
					Buttons.GoToPage (2);
				else if (res.category == SearchCategory.diary)
					Buttons.GoToPage (3);
				else
					Buttons.GoToPage (1);
			}
		}
		
		
		public void on_SearchList_row_activated (object o, RowActivatedArgs args)
		{
			on_SearchShowBtn_clicked (null, null);
		}
		
		
		public void on_SearchQuery_changed (object o, EventArgs args)
		{
			Gtk.Entry  SearchKeyword = (Gtk.Entry) o;
			Gtk.Button SearchBtn     = (Gtk.Button) gui.GetWidget ("SearchBtn");
			
			if (SearchKeyword.Text.Length > 0)
				SearchBtn.Sensitive = true;
			else
				SearchBtn.Sensitive = false;
				
		}
		
		
		public void on_SearchQuery_activate (object o, EventArgs args)
		{
			Gtk.Button SearchBtn = (Gtk.Button) gui.GetWidget ("SearchBtn");
			
			if (SearchBtn.Sensitive == true)
				on_SearchBtn_clicked (null, null);
		}
		
		
		public void on_SearchAllEntries_toggled (object o, EventArgs args)
		{
			Gtk.HBox SearchFromHBox = (Gtk.HBox) gui.GetWidget ("SearchFromHBox");
			Gtk.HBox SearchToHBox   = (Gtk.HBox) gui.GetWidget ("SearchToHBox");
			
			Gtk.CheckButton SearchAllEntries = (Gtk.CheckButton) gui.GetWidget ("SearchAllEntries");
			if (SearchAllEntries.Active == true) {
				SearchFromHBox.Sensitive = false;
				SearchToHBox.Sensitive   = false;
			} else {
				SearchFromHBox.Sensitive = true;
				SearchToHBox.Sensitive   = true;
			}
		}
		
		
		public void on_SearchList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeView SearchList    = (Gtk.TreeView) gui.GetWidget ("SearchList");
			Gtk.Button   SearchShowBtn = (Gtk.Button)   gui.GetWidget ("SearchShowBtn");
			
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SearchList.Selection.GetSelected (out mod, out itr))
				SearchShowBtn.Sensitive = true;
			else
				SearchShowBtn.Sensitive = false;
		}
		
		
		
		/* STATS **************************************************************/
		public void StatsCreate ()
		{
			Gtk.Image StatsIcon    = (Gtk.Image) gui.GetWidget ("StatsIcon");
			StatsIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Stats-20.png");
			
			Gtk.HBox StatsRangeHBox = (Gtk.HBox) gui.GetWidget ("StatsRangeHBox");
			StatsRangeHBox.PackStart (new Gtk.Label (Catalog.GetString ("From")), false, false, 6);
			StatsRangeHBox.PackStart (StatsFromDate, false, false, 6);
			StatsRangeHBox.PackStart (new Gtk.Label (Catalog.GetString ("To")), false, false, 6);
			StatsRangeHBox.PackStart (StatsToDate, false, false, 6);
			StatsRangeHBox.ShowAll ();
			
			
			Gtk.Viewport StatsAverageViewport = (Gtk.Viewport) gui.GetWidget ("StatsAverageViewport");
			StatsAverageViewport.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			
			
			Gtk.TreeViewColumn col = new Gtk.TreeViewColumn (Catalog.GetString ("Marks"), new CellRendererText (), "markup", 1);
			col.Expand = true;
			
			ListStore StatsListStore = new ListStore (typeof (string), typeof (string), typeof (string));
			StatsList.Model = StatsListStore;
			
			StatsList.AppendColumn (Catalog.GetString ("Subject"), new CellRendererText (), "markup", 0);
			StatsList.AppendColumn (col);
			
			CellRendererText rend = new CellRendererText ();
			rend.Xalign = (float) 1.00;
			StatsList.AppendColumn (Catalog.GetString ("Average"), rend, "markup", 2);
			
			StatsList.Columns [0].Clickable = true;
			StatsList.Columns [0].SortIndicator = true;
			StatsList.Columns [0].SortColumnId = 0;
			
			StatsList.Columns [2].Clickable = true;
			StatsList.Columns [2].SortIndicator = true;
			StatsList.Columns [2].SortColumnId = 2;
		}
		
		
		public void on_StatsCreateBtn_clicked (object o, EventArgs args)
		{
			this.Window.Sensitive = false;
			while (Application.EventsPending ())
                Application.RunIteration ();
			
			
			Gtk.CheckButton StatsIncludeMod = (Gtk.CheckButton) gui.GetWidget ("StatsIncludeMod");
			
			
			ArrayList list  = DB.CreateStats (StatsIncludeMod.Active, StatsFromDate.GetDate (), StatsToDate.GetDate ());
			ArrayList marks = new ArrayList ();
			ListStore store = (ListStore) StatsList.Model;
			
			store.Clear ();
			
			foreach (StatsResult item in list) {
				if (StatsCheckSubject (item.subject))
					store.AppendValues (new object [] {item.subject, item.Marks2ToString (), Decimal.Round ((Decimal) item.average, 2).ToString ()});
				
				marks.Add (item.average);
			}
			
			
			Gtk.Label StatsAverage = (Gtk.Label) gui.GetWidget ("StatsAverage");
			
			StatsAverage.Markup = "<b>" + Catalog.GetString ("Average") + " " + Decimal.Round (Convert.ToDecimal (DB.Average (marks)), 2) + "</b>";
			
			
			this.Window.Sensitive = true;
			while (Application.EventsPending ())
                Application.RunIteration ();
		}
		
		
		public void on_StatsSettingsBtn_clicked (object o, EventArgs args)
		{
			SettingsWindow wdw = new SettingsWindow ();
			wdw.Buttons.GoToPage (3);
			wdw.Window.Show ();
		}
		
		
		public bool StatsCheckSubject (string subject)
		{
			foreach (string tmp in Settings.Get ("SubjectsList").Split (';'))
				if (tmp == subject)
						return false;
		
			return true;
		}
		
		
		
		/* OTHER FUNCTIONS ****************************************************/
		public gnutu.Entry GetEntry ()
		{
			gnutu.Entry e = new gnutu.Entry ();
			
			// date
			e.date = new Date (this.Calendar.Date);
			
			// notes
			e.notes = AtSchoolNotes.Buffer.Text;
			
			// lessons
			e.lessons = new ArrayList ();
			
			Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
			Gtk.TreeIter  iter;
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				Lessons les = new Lessons ();
				
				les.time  = new Time ((string) store.GetValue (iter, 0), Settings.GetBool ("TimeMode24"));
				les.name  = (string) store.GetValue (iter, 1);
				les.marks = (string) store.GetValue (iter, 2);
				
				string test = (string) store.GetValue (iter, 3);
				
				if (test != "") {
					string ntest = test.Replace (EditSubjectWindow.TestColor, "");
					
					if (ntest == test) {
						ntest = test.Replace (EditSubjectWindow.QuizColor, "");
						les.mod = Test.quiz;
					} else
						les.mod = Test.test;
					
					ntest = ntest.Replace ("</span>", "");
					les.testdesc = ntest;
				} else {
					les.testdesc = "";
					les.mod = Test.none;
				}
				
				
				e.lessons.Add (les);
			}
			
			// diary
			e.diary = DiaryTextView.Buffer.Text;
			
			// tasks list
			e.todo = new ArrayList ();
			store  = (Gtk.ListStore) AtHomeTasksList.Model;
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				ToDo task = new ToDo ();
				
				task.did      = (bool)   store.GetValue (iter, 0);
				task.item     = (string) store.GetValue (iter, 2);
				task.Reminder = (bool)   store.GetValue (iter, 4);
				
				if (task.Reminder == true) {
					task.ReminderAt    = new Time ((string) store.GetValue (iter, 5), true);
					task.ReminderCount = (int)    store.GetValue (iter, 6);
				}
				
				if ((string) store.GetValue (iter, 3) == "1")
					task.priority = Priority.low;
				else if ((string) store.GetValue (iter, 3) == "2")
					task.priority = Priority.medium;
				else
					task.priority = Priority.high;
				
				e.todo.Add (task);
			}
			
			
			return e;
		}
		
		
		public void SetEntry (gnutu.Entry e)
		{
			ClearEntry (true, true, true, true);
			
			// notes
			AtSchoolNotes.Buffer.Text = e.notes;
			
			// diary
			DiaryTextView.Buffer.Text = e.diary;
			
			// lessons
			ListStore store = (ListStore) AtSchoolLessonsList.Model;
			
			foreach (Lessons les in e.lessons) {
				if (les.mod == Test.none)
					store.AppendValues (new object [] {les.time.ToString (Settings.GetBool ("TimeMode24")), les.name , les.marks, les.testdesc});
				else if (les.mod == Test.quiz)
					store.AppendValues (new object [] {les.time.ToString (Settings.GetBool ("TimeMode24")), les.name , les.marks, EditSubjectWindow.QuizColor + les.testdesc + "</span>"});
				else if (les.mod == Test.test)
					store.AppendValues (new object [] {les.time.ToString (Settings.GetBool ("TimeMode24")), les.name , les.marks, EditSubjectWindow.TestColor + les.testdesc + "</span>"});
			}
			
			// tasks list
			store = (ListStore) AtHomeTasksList.Model;
			
			foreach (ToDo task in e.todo) {
				if (task.priority == Priority.low)
					store.AppendValues (new object [] {task.did, new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-1.png")), task.item, "1", task.Reminder, task.ReminderAt.ToString (true), task.ReminderCount});
				else if (task.priority == Priority.medium)
					store.AppendValues (new object [] {task.did, new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-2.png")), task.item, "2", task.Reminder, task.ReminderAt.ToString (true), task.ReminderCount});
				else
					store.AppendValues (new object [] {task.did, new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-3.png")), task.item, "3", task.Reminder, task.ReminderAt.ToString (true), task.ReminderCount});
			}
			
			
			
			Gtk.Button RemoveSubject   = (Gtk.Button)       gui.GetWidget ("AtSchoolRemoveSubject");
			Gtk.Button RemoveSubject2  = (Gtk.Button)       gui.GetWidget ("AtHomeRemoveTask");
			Gtk.Button ModifySubject   = (Gtk.Button)       gui.GetWidget ("AtSchoolModifySubject");
			Gtk.Button ModifySubject2  = (Gtk.Button)       gui.GetWidget ("AtHomeModifyTask");
			Gtk.ToggleButton Completed = (Gtk.ToggleButton) gui.GetWidget ("AtHomeCompletedBtn");
			
			RemoveSubject.Sensitive = false;
			RemoveSubject2.Sensitive = false;
			ModifySubject.Sensitive = false;
			ModifySubject2.Sensitive = false;
			Completed.Sensitive = false;
		}
		
		
		public void LoadEntry (DateTime date, bool SavePrevious)
		{
			// Previous entry: save
			if (SavePrevious) {
				if (DB.EntryExists (CurrentDate))
					DB.ModifyEntry (GetEntry (), CurrentDate);
				else {
					if (GetEntry ().IsEmpty () == false)
						DB.AddEntry (GetEntry (), CurrentDate);
				}
			}
			
			
			// Selected entry: load (if exists) or create new
			if (DB.EntryExists (date))
				SetEntry (DB.GetEntryByDate (date));
			else {
				ClearEntry (true, true, true, true);
				
				// Load subjects
				if (Settings.GetBool ("AutomaticSubjectsInserting")) {
					if ((DaysOffSchool.IsDayOffSchool (new Date (date)) == false) &&
						(Settings.GetBool ("AutomaticSubjectsInserting") == true) &&
						(date >= new Date (Settings.Get ("SchoolYearBegin"), false).ToDateTime ()) &&
						(date <= new Date (Settings.Get ("SchoolYearEnd"), false).ToDateTime ()))
						InsertSubjects ();
				}
			}
			
			
			CurrentDate = date;
			SummaryCreate ();
		}
		
		
		public void ClearEntry (bool Notes, bool LessonsList, bool Diary, bool ToDo)
		{
			// Notes
			if (Notes == true)
				AtSchoolNotes.Buffer.Text = "";
			
			// LessonsList
			if (LessonsList == true) {
				Gtk.ListStore store = (Gtk.ListStore) AtSchoolLessonsList.Model;
				store.Clear ();
			}
			
			// Diary
			if (Diary == true)
				DiaryTextView.Buffer.Text = "";
			
			// ToDo
			if (ToDo == true) {
				Gtk.ListStore store = (Gtk.ListStore) AtHomeTasksList.Model;
				store.Clear ();
			}
		}
	}
	
	
	
	/*
	 *   EditSubjectWindow class
	 */
	 
	public class EditSubjectWindow {
		public static string TestColor = "<span color='#FF0000'>";
		public static string QuizColor = "<span color='#ec9000'>";
		
		private Glade.XML     gui;
		private Gtk.TreeModel model;
		private TimeEdit      Time;
		private bool          NewEntry;
		private Gtk.TreeIter  Iter;
		
		[Glade.Widget] public Gtk.Window Window;
		[Glade.Widget] public Gtk.Table  WidgetsTable;
		
		
		public EditSubjectWindow (Gtk.TreeModel model)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "EditSubjectWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("EditSubjectWindow");
			CreateCompletion ();
			
			Time = new TimeEdit ();
			WidgetsTable.Attach (Time, 0, 1, 1, 2);
			WidgetsTable.ShowAll ();
			
			NewEntry = true;
			
			this.model = model;
		}
		
		
		public EditSubjectWindow (Gtk.TreeModel model, string time, string subject, string marks, string test)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "EditSubjectWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("EditSubjectWindow");
			CreateCompletion ();
			
			Time = new TimeEdit ();
			Time.SetTime (time);
			WidgetsTable.Attach (Time, 0, 1, 1, 2);
			WidgetsTable.ShowAll ();
			
			
			// Loading selected subject
			Gtk.Entry       Subject         = (Gtk.Entry) gui.GetWidget ("Subject");
			Gtk.Entry       Marks           = (Gtk.Entry) gui.GetWidget ("Marks");
			Gtk.Entry       TestDescription = (Gtk.Entry) gui.GetWidget ("TestDescription");
			Gtk.RadioButton OptionNone      = (Gtk.RadioButton) gui.GetWidget ("OptionNone");
			Gtk.RadioButton OptionQuiz      = (Gtk.RadioButton) gui.GetWidget ("OptionQuiz");
			Gtk.RadioButton OptionTest      = (Gtk.RadioButton) gui.GetWidget ("OptionTest");
			
			Subject.Text = subject;
			Marks.Text   = marks;
			
			if (test != "") {
				string Ntest = test.Replace (TestColor, "");
				
				if (Ntest == test) {
					Ntest = test.Replace (QuizColor, "");
					OptionQuiz.Active = true;
				} else
					OptionTest.Active = true;
				
				Ntest = Ntest.Replace ("</span>", "");
				TestDescription.Text = Ntest;
			} else
				OptionNone.Active = true;
			
			
			// iter to remove 
			Gtk.ListStore store = (Gtk.ListStore) model;
			Gtk.TreeIter  iter;
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
				if (time == (string) store.GetValue (iter, 0))
					Iter = iter;
			}
			
			
			NewEntry = false;
			this.model = model;
		}
		
		
		public void on_Ok_clicked (object o, EventArgs args)
		{
			Gtk.Entry       Subject         = (Gtk.Entry) gui.GetWidget ("Subject");
			Gtk.Entry       Marks           = (Gtk.Entry) gui.GetWidget ("Marks");
			Gtk.Entry       TestDescription = (Gtk.Entry) gui.GetWidget ("TestDescription");
			Gtk.RadioButton OptionNone      = (Gtk.RadioButton) gui.GetWidget ("OptionNone");
			Gtk.RadioButton OptionQuiz      = (Gtk.RadioButton) gui.GetWidget ("OptionQuiz");
			Gtk.ListStore   store           = (Gtk.ListStore) model;
			
			TestDescription.Text = TestDescription.Text.Replace ("<", "");
			TestDescription.Text = TestDescription.Text.Replace (">", "");
			

			// checking time
			Gtk.TreeIter iter;
			
			if (NewEntry == true) {
				for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter)) {
					if (Time.GetText () == (string) store.GetValue (iter, 0)) {
							Gtk.MessageDialog dialog = new Gtk.MessageDialog (Window,
													DialogFlags.DestroyWithParent,
													MessageType.Warning,
													ButtonsType.Close,
													Catalog.GetString ("Subject with this time exists !\nEnter other time."));
							
							dialog.Run ();
							dialog.Destroy ();
							
							this.Window.Focus = Time.Hour;
							
							return;
					}
				}
			} else
				store.Remove (ref Iter);
			
			
			// checking entries
			if ((Subject.Text == "") ||
				((OptionNone.Active == false) && (TestDescription.Text == ""))) {
				
				string msg;
				if (Subject.Text == "") {
					msg = Catalog.GetString ("Subject entry mustn't be empty !");
					this.Window.Focus = Subject;
				} else {
					msg = Catalog.GetString ("Test description entry mustn't be empty !");
					this.Window.Focus = TestDescription;
				}
				
				Gtk.MessageDialog dialog = new Gtk.MessageDialog (Window,
											DialogFlags.DestroyWithParent,
											MessageType.Warning,
											ButtonsType.Close, msg);
				
				dialog.Run ();
				dialog.Destroy ();
				
				return;
			}
			
			
			// Adding new Subject
			if (OptionNone.Active == true)
				store.AppendValues (new object [] {Time.GetText (), Subject.Text, Marks.Text, ""});
			else if (OptionQuiz.Active == true)
				store.AppendValues (new object [] {Time.GetText (), Subject.Text, Marks.Text, QuizColor + TestDescription.Text + "</span>"});
			else
				store.AppendValues (new object [] {Time.GetText (), Subject.Text, Marks.Text, TestColor + TestDescription.Text + "</span>"});
			
			Window.Destroy ();
		}
		
		
		public void on_Cancel_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void on_OptionNone_toggled (object o, EventArgs args)
		{
			Gtk.RadioButton btn     = (Gtk.RadioButton) o;
			Gtk.HBox        TestBox = (Gtk.HBox) gui.GetWidget ("TestBox");
			
			if (btn.Active == true)
				TestBox.Sensitive = false;
			else
				TestBox.Sensitive = true;
		}
		
		
		public void on_Subject_activate (object o, EventArgs args)
		{
			on_Ok_clicked (null, null);
		}
		
		
		public void on_Marks_activate (object o, EventArgs args)
		{
			on_Ok_clicked (null, null);
		}
		
		
		public void on_TestDescription_activate (object o, EventArgs args)
		{
			on_Ok_clicked (null, null);
		}
		
		
		private void CreateCompletion ()
		{
			Gtk.Entry           subject = (Gtk.Entry) gui.GetWidget ("Subject");
			Gtk.EntryCompletion comp    = new Gtk.EntryCompletion ();
			Gtk.ListStore       mod     = new Gtk.ListStore (typeof (string));
			
			comp.Model = mod;
			comp.TextColumn = 0;
			subject.Completion = comp;
			

			ArrayList list = LessonsScheduleData.GetLessonsList ();
			
			foreach (string item in list)
				mod.AppendValues (new object [] {item});
		}
	}
	


	/*
	 *  SearchingTestsWindow class
	 */
	public class SearchingTestsWindow {
		private Glade.XML     gui;
		
		[Glade.Widget] public  Gtk.Window   Window;
		[Glade.Widget] private Gtk.HBox     SearchFromBox;
		[Glade.Widget] private Gtk.HBox     SearchToBox;
		[Glade.Widget] private Gtk.Label    Tests;
		[Glade.Widget] private Gtk.Viewport TestsViewport;

		private DateEdit From;
		private DateEdit To;
		
		
		public SearchingTestsWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "SearchingTestsWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("SearchingTestsWindow");
			
			Gtk.Viewport HeadBg = (Gtk.Viewport) gui.GetWidget ("HeadBg");
			HeadBg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			TestsViewport.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));

	
			// HeadIcon
			Gtk.Image HeadIcon = (Gtk.Image) gui.GetWidget ("HeadIcon");
			HeadIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "SearchTests-48.png");
			
			// SearchBtnIcon
			Gtk.Image SearchBtnIcon = (Gtk.Image) gui.GetWidget ("SearchBtnIcon");
			SearchBtnIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Search-20.png");
			
			// DateEdits
			From = new DateEdit ();
			From.Show ();
			SearchFromBox.Add (From);
			
			DateTime date = From.GetDate ();
			date = date.AddDays (40);
			
			To = new DateEdit ();
			To.SetDate (date);
			To.Show ();
			SearchToBox.Add (To);
		}


		public void on_SearchTests_toggled (object o, EventArgs args)
		{
			Gtk.Button      SearchBtn    = (Gtk.Button) gui.GetWidget ("SearchBtn");
			Gtk.Expander    More         = (Gtk.Expander) gui.GetWidget ("More");
			Gtk.CheckButton SearchQuizes = (Gtk.CheckButton) gui.GetWidget ("SearchQuizes");
			Gtk.CheckButton SearchTests  = (Gtk.CheckButton) o;

			if ((SearchQuizes.Active == false) && (SearchTests.Active == false)) {
				SearchBtn.Sensitive = false;
				More.Sensitive      = false;
			} else {
				SearchBtn.Sensitive = true;
				More.Sensitive      = true;
			}
		}
		

		public void on_SearchQuizes_toggled (object o, EventArgs args)
		{
			Gtk.Button      SearchBtn    = (Gtk.Button) gui.GetWidget ("SearchBtn");
			Gtk.Expander    More         = (Gtk.Expander) gui.GetWidget ("More");
			Gtk.CheckButton SearchTests  = (Gtk.CheckButton) gui.GetWidget ("SearchTests");
			Gtk.CheckButton SearchQuizes = (Gtk.CheckButton) o;

			if ((SearchQuizes.Active == false) && (SearchTests.Active == false)) {
				SearchBtn.Sensitive = false;
				More.Sensitive      = false;
			} else {
				SearchBtn.Sensitive = true;
				More.Sensitive      = true;
			}
		}
		

		public void on_SearchBtn_clicked (object o, EventArgs args)
		{
			Tests.Markup = "";
			this.Window.Sensitive = false;
			while (Application.EventsPending ())
	        		Application.RunIteration ();

			string text = "";
			
			Gtk.CheckButton SearchTests  = (Gtk.CheckButton) gui.GetWidget ("SearchTests");
			Gtk.CheckButton SearchQuizes = (Gtk.CheckButton) gui.GetWidget ("SearchQuizes");
			
			ArrayList tests = DB.SearchTests (From.GetDate (), To.GetDate (), 
						SearchTests.Active, SearchQuizes.Active);
			
			foreach (SearchTestsResult res in tests) {
				text += "<b>" + res.date.ToString (Settings.GetBool ("DateFirstYear"));
				text += " "   + res.mod;
				text += " - " + res.name + "</b>\n";
				text += "<i>" + res.desc + "</i>\n\n";
			}

			if (text == "")
				text = "<b>" + Catalog.GetString ("No tests and/or quizes found") + "</b>";

			
			Tests.Markup = text;

			this.Window.Sensitive = true;
			while (Application.EventsPending ())
	        		Application.RunIteration ();
		}
		
		
		public void on_ExitBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}

	}
	
	
	
	/*
	 *  AboutWindow class
	 */
	
	public class AboutWindow {
		private Glade.XML     gui;
		
		[Glade.Widget] public Gtk.Window Window;
		[Glade.Widget] public Gtk.Label  VersionLabel;
		
		
		public AboutWindow ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "AboutWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("AboutWindow");
			
			
			Gtk.Viewport HeadBg = (Gtk.Viewport) gui.GetWidget ("HeadBg");
			HeadBg.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			
			Gtk.HBox  LogoBox = (Gtk.HBox) gui.GetWidget ("LogoBox");
			Gtk.Image LogoImg = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "logo-48.png"));
			LogoImg.Show ();
			LogoBox.Add (LogoImg);
			
			// Version
			VersionLabel.Text = Catalog.GetString ("version") + " " + gnutu.Defines.Version;
		}
		
		
		public void on_ExitBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
	}
	
	
	
	/*
	 *  ModifyTaskWindow class
	 */
	
	public class ModifyTaskWindow {
		private Glade.XML     gui;
		private bool          modify;
		private Gtk.ListStore store;
		private Gtk.TreeIter  iter;
		private TimeEdit      AtTime;
		
		[Glade.Widget] public  Gtk.Window      Window;
		[Glade.Widget] private Gtk.Entry       Task;
		[Glade.Widget] private Gtk.ComboBox    Priority;
		[Glade.Widget] private Gtk.Button      OkBtn;
		[Glade.Widget] private Gtk.HBox        ReminderHBox0, ReminderHBox2, ReminderHBox3;
		[Glade.Widget] private Gtk.CheckButton Reminder;
		[Glade.Widget] private Gtk.SpinButton  ReminderMinutes;
		[Glade.Widget] private Gtk.Expander    MoreExpander;
		
		
		public ModifyTaskWindow (Gtk.ListStore store)
		{
			ModifyTaskWindowCreate ();
			
			this.AtTime.SetTime (DateTime.Now, Settings.GetBool ("TimeMode24"));
			
			this.modify = false;
			this.store  = store;
			this.Priority.Active = 1;
			this.MoreExpander.Expanded = false;
		}
		
		
		public ModifyTaskWindow (Gtk.ListStore store, Gtk.TreeIter iter)
		{
			ModifyTaskWindowCreate ();
			
			this.modify = true;
			this.store  = store;
			this.iter   = iter;
			
			this.Priority.Active = 3 - Convert.ToInt32 ((string) store.GetValue (iter, 3));
			this.Task.Text       = (string) store.GetValue (iter, 2);
			this.Reminder.Active = (bool) store.GetValue (iter, 4);
			if (Reminder.Active == true) {
				this.AtTime.SetTime (new Time ((string) store.GetValue (iter, 5), true).ToDateTime (), Settings.GetBool ("TimeMode24"));
				this.ReminderMinutes.Value = (int) store.GetValue (iter, 6);
				ReminderHBox0.Sensitive = true;
				this.MoreExpander.Expanded = true;
			} else {
				ReminderHBox0.Sensitive = false;
				this.AtTime.SetTime (new Time (DateTime.Now).ToDateTime (), Settings.GetBool ("TimeMode24"));
				this.MoreExpander.Expanded = false;
			}
		}
		
		
		public void ModifyTaskWindowCreate ()
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "ModifyTaskWindow", null);
			gui.Autoconnect (this);
			Window = (Gtk.Window) gui.GetWidget ("ModifyTaskWindow");
			
			AtTime = new TimeEdit ();
			AtTime.SetTime (DateTime.Today, Settings.GetBool ("TimeMode24"));
			
			ReminderHBox2.Add (AtTime);
		}
		
		
		public void on_Task_activate (object o, EventArgs args)
		{
			if ((Task.Text.Length > 0) && (this.Priority.Active > -1))
				on_OkBtn_clicked (null, null);
		}
		
		
		public void on_Task_changed (object o, EventArgs args)
		{
			if ((Task.Text.Length > 0) && (this.Priority.Active > -1))
				OkBtn.Sensitive = true;
			else
				OkBtn.Sensitive = false;
		}
		
		
		public void on_Priority_changed (object o, EventArgs args)
		{
			if ((Task.Text.Length > 0) && (this.Priority.Active > -1))
				OkBtn.Sensitive = true;
			else
				OkBtn.Sensitive = false;
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			int priority = 3 - this.Priority.Active;
			
			if (modify == true) {
				store.SetValue (iter, 1, new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-" + priority.ToString () + ".png")));
				store.SetValue (iter, 2, Task.Text);
				store.SetValue (iter, 3, priority.ToString ());
				store.SetValue (iter, 4, Reminder.Active);
				store.SetValue (iter, 5, AtTime.GetTime (true));
				store.SetValue (iter, 6, (int) ReminderMinutes.Value);
			} else
				store.AppendValues (new object [] {false,
								new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("Priority-" + priority.ToString () + ".png")),
								Task.Text, priority.ToString (),
								Reminder.Active,
								AtTime.GetTime (true), (int) ReminderMinutes.Value});
			
			Window.Destroy ();
		}
		
		
		public void on_Reminder_toggled (object o, EventArgs args)
		{
			ReminderHBox0.Sensitive = Reminder.Active;
			ReminderHBox3.Sensitive = Reminder.Active;
		}
	}
}
