/*
 *  SettingsWindow.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Gtk;
using Glade;
using gnutu.Widgets;
using Mono.Unix;


namespace gnutu.GUI {
	public class SettingsWindow {
		private Glade.XML wdw;
		
		[Glade.Widget] public  Gtk.Window   Window;
		[Glade.Widget] public  Gtk.VBox     ButtonsSwitcherPanel;
		[Glade.Widget] public  Gtk.HBox     LessonsStartHBox;
		[Glade.Widget] public  Gtk.Notebook Pages;
		[Glade.Widget] private Gtk.Table    GeneralTable;
		[Glade.Widget] private Gtk.TreeView SubjectsList;
		[Glade.Widget] private Gtk.Entry    SubjectName;
		[Glade.Widget] private Gtk.Button   RemoveSubjectBtn;
		[Glade.Widget] private Gtk.Button   AddSubjectBtn;
		
		public  ButtonsSwitcher Buttons;
		
		private DateEdit SchoolYearBegin;
		private DateEdit SchoolYearEnd;
		private DateEdit HolidayEnd;
		private TimeEdit LessonsStartAt;
		
		
		public SettingsWindow ()
		{
			wdw = Glade.XML.FromAssembly ("gui.glade", "SettingsWindow", null);
			wdw.Autoconnect (this);
			Window = (Gtk.Window) wdw.GetWidget ("SettingsWindow");
			
			Pages.ShowTabs = false;
			
			
			// buttons switcher
			Buttons = new ButtonsSwitcher (Pages, true);
			ButtonsSwitcherPanel.PackStart (Buttons, false, true, 3);
			Buttons.Show ();
			
			Buttons.Add (Catalog.GetString ("General settings"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Settings-20.png")));
			Buttons.AddSeparator ();
			Buttons.Add (Catalog.GetString ("Summary"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Day-20.png")));
			Buttons.Add (Catalog.GetString ("At school"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtSchool-20.png")));
			Buttons.Add (Catalog.GetString ("Stats"), new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Stats-20.png")));
			
			
			// date edits
			SchoolYearBegin = new DateEdit ();
			SchoolYearEnd   = new DateEdit ();
			HolidayEnd      = new DateEdit ();
			
			GeneralTable.Attach (SchoolYearBegin, 1, 2, 0, 1);
			GeneralTable.Attach (SchoolYearEnd, 1, 2, 1, 2);
			GeneralTable.Attach (HolidayEnd, 1, 2, 2, 3);
			
			
			// time edits
			LessonsStartAt = new TimeEdit ();
			LessonsStartHBox.PackStart (LessonsStartAt, false, false, 6);
			
			
			// set tabs icons
			Gtk.Image GeneralIcon = (Gtk.Image) wdw.GetWidget ("GeneralIcon");
			GeneralIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Settings-20.png");
			
			Gtk.Image SummaryIcon = (Gtk.Image) wdw.GetWidget ("SummaryIcon");
			SummaryIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Day-20.png");
			
			Gtk.Image AtSchoolIcon = (Gtk.Image) wdw.GetWidget ("AtSchoolIcon");
			AtSchoolIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "AtSchool-20.png");
			
			Gtk.Image StatsIcon = (Gtk.Image) wdw.GetWidget ("StatsIcon");
			StatsIcon.Pixbuf = new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Stats-20.png");
			
			
			// subjects list
			Gtk.ListStore store = new Gtk.ListStore (typeof (string));
			SubjectsList.Model = store;
			
			SubjectsList.AppendColumn ("", new CellRendererText (), "text", 0);
			store.SetSortColumnId (0, Gtk.SortType.Ascending);
			
			
			// completion
			Gtk.EntryCompletion comp = new Gtk.EntryCompletion ();
			Gtk.ListStore       mod  = new Gtk.ListStore (typeof (string));
			
			comp.Model = mod;
			comp.TextColumn = 0;
			SubjectName.Completion = comp;
			

			ArrayList list = LessonsScheduleData.GetLessonsList ();
			
			foreach (string item in list)
				mod.AppendValues (new object [] {item});
		
			
			// Loading settings
			LoadSettings ();
		}
		
		
		public void on_SettingsWindow_delete_event (object o, DeleteEventArgs args) 
		{
			args.RetVal = false;
		}
		
		
		public void on_SubjectName_changed (object o, EventArgs args)
		{
			if (SubjectName.Text.Length > 0)
				AddSubjectBtn.Sensitive = true;
			else
				AddSubjectBtn.Sensitive = false;
		}
		
		
		public void on_SubjectsList_cursor_changed (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SubjectsList.Selection.GetSelected (out mod, out itr))
				RemoveSubjectBtn.Sensitive = true;
			else
				RemoveSubjectBtn.Sensitive = false;
		}
		
		
		public void on_SubjectName_activate (object o, EventArgs args)
		{
			if (SubjectName.Text.Length > 0)
				on_AddSubjectBtn_clicked (null, null);
		}
		
		
		public void on_AddSubjectBtn_clicked (object o, EventArgs args)
		{
			Gtk.ListStore store = (Gtk.ListStore) SubjectsList.Model;
			
			Gtk.TreeIter iter;
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter))
				if ((string) store.GetValue (iter, 0) == SubjectName.Text)
					return;
			
			store.AppendValues (new object [] {SubjectName.Text});
			SubjectName.Text = "";
		}
		
		
		public void on_RemoveSubjectBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeModel mod;
			Gtk.TreeIter  itr;
			
			if (SubjectsList.Selection.GetSelected (out mod, out itr)) {
				Gtk.ListStore store = (Gtk.ListStore) SubjectsList.Model;
				store.Remove (ref itr);
				
				RemoveSubjectBtn.Sensitive = false;
			}
		}
		
		
		public void on_OkBtn_clicked (object o, EventArgs args)
		{
			SaveSettings ();
			Window.Destroy ();
		}
		
		
		public void on_CancelBtn_clicked (object o, EventArgs args)
		{
			Window.Destroy ();
		}
		
		
		public void LoadSettings ()
		{
			// General settings
			SchoolYearBegin.SetDate (new Date (Settings.Get ("SchoolYearBegin"), false).ToDateTime ());
			SchoolYearEnd.SetDate   (new Date (Settings.Get ("SchoolYearEnd"), false).ToDateTime ());
			HolidayEnd.SetDate      (new Date (Settings.Get ("HolidayEnd"), false).ToDateTime ());
			
			SetBool ("TimeMode24");
			SetBool ("DateFirstYear");
			SetBool ("MarkDaysOffSchool");
			SetBool ("MinimalizeWindowToTray");
			SetBoolDF ("ShowOnlyTrayIcon");
			SetBool ("InformationsAboutUpdates");
			
			// Summary
			SetBool ("ShowTestAndQuizes");
			SetBool ("ShowNotes");
			SetBool ("ShowToDoList");
			SetBool ("ShowCompletedTasks");
			SetBool ("ShowDiary");
			SetBool ("ShowLessonsList");
			SetBool ("ShowDaysLeft");
			SetBool ("ShowNearestTests");
			
			// AtSchool
			SetBool ("AutomaticSubjectsInserting");
			LessonsStartAt.SetTime  (new Time (Settings.Get ("LessonsStartAt"), true).ToDateTime (), Settings.GetBool ("TimeMode24"));
			
			// Stats
			SetFloat ("PlusValue");
			SetFloat ("MinusValue");
			SetList  ("SubjectsList");
		}
		
		
		public void SaveSettings ()
		{
			// General settings
			Settings.Set ("SchoolYearBegin", SchoolYearBegin.GetDate2 ().ToString (false));
			Settings.Set ("SchoolYearEnd", SchoolYearEnd.GetDate2 ().ToString (false));
			Settings.Set ("HolidayEnd", HolidayEnd.GetDate2 ().ToString (false));
			
			GetBool ("TimeMode24");
			GetBool ("DateFirstYear");
			GetBool ("MarkDaysOffSchool");
			GetBool ("MinimalizeWindowToTray");
			GetBool ("ShowOnlyTrayIcon");
			GetBool ("InformationsAboutUpdates");
			
			// Summary
			GetBool ("ShowTestAndQuizes");
			GetBool ("ShowNotes");
			GetBool ("ShowToDoList");
			GetBool ("ShowCompletedTasks");
			GetBool ("ShowDiary");
			GetBool ("ShowLessonsList");
			GetBool ("ShowDaysLeft");
			GetBool ("ShowNearestTests");
			
			// AtSchool
			GetBool ("AutomaticSubjectsInserting");
			Settings.Set ("LessonsStartAt", LessonsStartAt.GetTime (true));
			
			// Stats
			GetFloat ("PlusValue");
			GetFloat ("MinusValue");
			GetList  ("SubjectsList");
		}
		
		
		public void GetBool (string WidgetName)
		{
			Gtk.CheckButton btn = (Gtk.CheckButton) wdw.GetWidget (WidgetName);
			Settings.SetBool (WidgetName, btn.Active);
		}
		
		
		public void SetBool (string WidgetName)
		{
			Gtk.CheckButton btn = (Gtk.CheckButton) wdw.GetWidget (WidgetName);
			btn.Active = Settings.GetBool (WidgetName);
		}
		
		
		public void SetBoolDF (string WidgetName)
		{
			Gtk.CheckButton btn = (Gtk.CheckButton) wdw.GetWidget (WidgetName);
			btn.Active = Settings.GetBoolDF (WidgetName);
		}
		
		
		public void GetFloat (string WidgetName)
		{
			Gtk.SpinButton btn = (Gtk.SpinButton) wdw.GetWidget (WidgetName);
			Settings.Set (WidgetName, btn.Value.ToString ());
		}
		
		
		public void SetFloat (string WidgetName)
		{
			Gtk.SpinButton btn = (Gtk.SpinButton) wdw.GetWidget (WidgetName);
			
			try {
				btn.Value = (float) Convert.ToDecimal (Settings.Get (WidgetName));
			} catch {
				btn.Value = (float) 0.00;
			}
		}
		
		
		public void GetInt (string WidgetName)
		{
			Gtk.SpinButton btn = (Gtk.SpinButton) wdw.GetWidget (WidgetName);
			Settings.Set (WidgetName, btn.Value.ToString ());
		}
		
		
		public void SetInt (string WidgetName)
		{
			Gtk.SpinButton btn = (Gtk.SpinButton) wdw.GetWidget (WidgetName);
			
			try {
				btn.Value = (int) Convert.ToInt32 (Settings.Get (WidgetName));
			} catch {
				btn.Value = 0;
			}
		}
		
		
		public void GetList (string ListName)
		{
			Gtk.ListStore store = (Gtk.ListStore) SubjectsList.Model;
			
			Gtk.TreeIter iter;
			string      str = "";
			
			for (bool tmp = store.GetIterFirst (out iter); tmp == true; tmp = store.IterNext (ref iter))
				str += (string) store.GetValue (iter, 0) + ";";
			
			Settings.Set (ListName, str);
		}
		
		
		public void SetList (string ListName)
		{
			Gtk.ListStore store = (Gtk.ListStore) SubjectsList.Model;
			
			string[] str = Settings.Get (ListName).Split (';');
			
			foreach (string tmp in str)
				if (tmp.Length > 0)
					store.AppendValues (new object [] {tmp});
		}
	}
}
