/*
 *  Alert.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using Mono.Unix;
using GLib;
using gnutu;


namespace gnutu.Widgets {

	public enum AlertType {
		Info,
		Question,
		Error,
		Warning
	}
	
	
	public class Alert : Gtk.Viewport {
		public Alert (AlertType type, string title, string message)
		{
		    CreateGUI (type, title, message, true);
		}
		
		
		public Alert (AlertType type, string message)
		{
		    CreateMiniGUI (type, message);
		}


		public Alert (AlertType type, string title, string message, bool ShowCloseButton)
		{
		    CreateGUI (type, title, message, ShowCloseButton);
		}


		public void CreateGUI (AlertType type, string title, string message, bool ShowCloseButton)
		{
			this.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			
			Gtk.HBox WindowBox = new Gtk.HBox ();
			WindowBox.BorderWidth = 12;
			this.Add (WindowBox);
			
			Gtk.Image icon = new Gtk.Image ();
			
			switch (type) {
				case AlertType.Error   : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Error-48.png"));    break;
				case AlertType.Info    : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Info-48.png"));     break;
				case AlertType.Question: icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Question-48.png")); break;
				case AlertType.Warning : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Warning-48.png"));  break;
			}
			
			icon.Yalign = (float) 0.00;
			
			WindowBox.PackStart (icon, false, true, 12);
			
			Gtk.VBox MessageBox = new Gtk.VBox ();
			MessageBox.Spacing = 6;
			WindowBox.PackEnd (MessageBox, true, true, 0);
			
			Gtk.Label TitleLabel = new Gtk.Label ();
			TitleLabel.UseMarkup = true;
			TitleLabel.Wrap      = true;
			TitleLabel.Xalign    = (float) 0.00;
			TitleLabel.Markup    = "<span weight=\"bold\" size=\"larger\">" + title + "</span>";

			Gtk.Label MessageLabel    = new Gtk.Label ();
			MessageLabel.UseMarkup    = true;
			MessageLabel.Markup       = message;
			MessageLabel.Xalign       = (float) 0.00;
			MessageLabel.Wrap         = false;
			

			MessageBox.Add (TitleLabel);
			MessageBox.Add (MessageLabel);

			
			if (ShowCloseButton == true) {
			    Gtk.HButtonBox ButtonsBox = new Gtk.HButtonBox ();
			    ButtonsBox.Layout         = Gtk.ButtonBoxStyle.End;
			    ButtonsBox.Spacing        = 6;

			    Gtk.Button ExitBtn      = new Gtk.Button ();
			    Gtk.Label  ExitBtnLabel = new Gtk.Label ();
			    ExitBtnLabel.UseMarkup  = true;
			    ExitBtnLabel.Markup     = "<span color=\"#1111EE\">" + Catalog.GetString ("Close") + "</span>";
			    ExitBtn.Relief          = Gtk.ReliefStyle.None;
			    ExitBtn.Clicked        += new EventHandler (on_ExitBtn_clicked);
			    ExitBtn.Add (ExitBtnLabel);
			    ButtonsBox.Add (ExitBtn);

			    MessageBox.Add (ButtonsBox);
			}
			
			this.ShowAll ();
		}
		
		
		public void CreateMiniGUI (AlertType type, string message)
		{
			this.ModifyBg (Gtk.StateType.Normal, new Gdk.Color (255, 255, 255));
			
			Gtk.HBox WindowBox = new Gtk.HBox ();
			WindowBox.BorderWidth = 8;
			this.Add (WindowBox);
			
			Gtk.Image icon = new Gtk.Image ();
			
			switch (type) {
				case AlertType.Error   : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Error-24.png"));    break;
				case AlertType.Info    : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Info-24.png"));     break;
				case AlertType.Question: icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Question-24.png")); break;
				case AlertType.Warning : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Warning-24.png"));  break;
			}
			
			icon.Yalign = (float) 0.00;
			
			WindowBox.PackStart (icon, false, true, 12);
			
			Gtk.Label MessageLabel = new Gtk.Label ();
			MessageLabel.UseMarkup = true;
			MessageLabel.Xalign    = (float) 0.00;
			MessageLabel.Markup    = "<span weight=\"bold\" size=\"larger\">" + message + "</span>";
			WindowBox.Add (MessageLabel);

			Gtk.HButtonBox ButtonsBox = new Gtk.HButtonBox ();
			ButtonsBox.Layout         = Gtk.ButtonBoxStyle.End;
			ButtonsBox.Spacing        = 6;

			Gtk.Button ExitBtn      = new Gtk.Button ();
			Gtk.Label  ExitBtnLabel = new Gtk.Label ();
			ExitBtnLabel.UseMarkup  = true;
			ExitBtnLabel.Markup     = "<span color=\"#1111EE\">" + Catalog.GetString ("Close") + "</span>";
			ExitBtn.Relief          = Gtk.ReliefStyle.None;
			ExitBtn.Clicked        += new EventHandler (on_ExitBtn_clicked);
			ExitBtn.Add (ExitBtnLabel);
			ButtonsBox.Add (ExitBtn);
			
			WindowBox.PackEnd (ButtonsBox, false, true, 12);
			

			this.ShowAll ();
		}
		

		private void on_ExitBtn_clicked (object o, EventArgs args)
		{
			this.Destroy ();
		}
	}
}
