/*
 *  TrayMessage.cs
 *  Copyright (C) 2004-2005 Marcin Krystian Krzywonos
 *  License: GNU/GPL, version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using GLib;
using gnutu;


namespace gnutu.Widgets {
	public enum MsgType {
		Info,
		Question,
		Error,
		Warning
	}
	
	
	public class TrayMessage {
		public  Gtk.Window    window;
		
		private Gtk.ListStore TasksListStore;
		private Gtk.TreeIter  TaskIter;
		private bool          reminder;
		private DateTime      EntryDate;
		private int           TaskID;
		
		private Gtk.Viewport view;
		
		
		public TrayMessage (MsgType type, bool ShowCloseBtn, Gtk.Button btn1, Gtk.Button btn2, string title, string message)
		{
			this.reminder = false;
			
			window = new Gtk.Window (Gtk.WindowType.Toplevel);
			window.TypeHint = Gdk.WindowTypeHint.Normal;
			window.Stick ();
			window.KeepAbove = true;
			window.SkipTaskbarHint = true;
			window.Decorated = false;
			
			view = new Gtk.Viewport ();
			view.ModifyBg (Gtk.StateType.Normal , new Gdk.Color (255, 255, 255));
			window.Add (view);
			
			Gtk.HBox WindowBox = new Gtk.HBox ();
			WindowBox.Spacing = 12;
			WindowBox.BorderWidth = 12;
			view.Add (WindowBox);
			
			Gtk.Image icon = new Gtk.Image ();
			switch (type) {
				case MsgType.Error   : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Error-48.png"));    break;
				case MsgType.Info    : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Info-48.png"));     break;
				case MsgType.Question: icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Question-48.png")); break;
				case MsgType.Warning : icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Warning-48.png"));  break;
			}
			WindowBox.Add (icon);
			
			Gtk.VBox LabelsBox = new Gtk.VBox ();
			LabelsBox.Spacing  = 6;
			WindowBox.PackStart (LabelsBox, false, true, 0);
			
			if (title.Length > 0) {
				Gtk.Label TitleLabel = new Gtk.Label ();
				TitleLabel.UseMarkup = true;
				TitleLabel.Wrap      = true;
				TitleLabel.Xalign    = (float) 0.00;
				TitleLabel.Markup    = "<span weight=\"bold\" size=\"larger\">" + title + "</span>";
				TitleLabel.WidthRequest = 300;
				LabelsBox.Add (TitleLabel);
			}
			
			if (message.Length > 0) {
				Gtk.Label MessageLabel    = new Gtk.Label ();
				MessageLabel.UseMarkup    = true;
				MessageLabel.Markup       = message;
				MessageLabel.Xalign       = (float) 0.00;
				MessageLabel.Wrap         = true;
				MessageLabel.WidthRequest = 300;
				LabelsBox.Add (MessageLabel);
			}
			
			Gtk.HButtonBox ButtonsBox = new Gtk.HButtonBox ();
			ButtonsBox.Layout         = Gtk.ButtonBoxStyle.End;
			ButtonsBox.Spacing        = 6;
			LabelsBox.Add (ButtonsBox);
			
			if (btn1 != null) {
				btn1.Relief = Gtk.ReliefStyle.None;
				ButtonsBox.Add (btn1);
			}
			
			if (btn2 != null) {
				btn2.Relief = Gtk.ReliefStyle.None;
				ButtonsBox.Add (btn2);
			}
			
			if (ShowCloseBtn == true) {
				Gtk.Button ExitBtn      = new Gtk.Button ();
				Gtk.Label  ExitBtnLabel = new Gtk.Label ();
				ExitBtnLabel.UseMarkup  = true;
				ExitBtnLabel.Markup     = "<span color=\"#1111EE\">" + Catalog.GetString ("Close") + "</span>";
				ExitBtn.Relief          = Gtk.ReliefStyle.None;
				ExitBtn.Clicked        += new EventHandler (on_ExitBtn_clicked);
				ExitBtn.Add (ExitBtnLabel);
				ButtonsBox.Add (ExitBtn);
			}
			
			WindowBox.ShowAll ();
		}
		
		
		public TrayMessage (string message, Gdk.Pixbuf priority, Gtk.ListStore TasksListStore, Gtk.TreeIter TaskIter)
		{
			EntryDate = DateTime.Today;
			
			window = new Gtk.Window (Gtk.WindowType.Toplevel);
			window.TypeHint = Gdk.WindowTypeHint.Normal;
			window.Stick ();
			window.KeepAbove = true;
			window.SkipTaskbarHint = true;
			window.Decorated = false;
			
			Gtk.Viewport view = new Gtk.Viewport ();
			view.ModifyBg (Gtk.StateType.Normal , new Gdk.Color (255, 255, 255));
			window.Add (view);
			
			Gtk.HBox WindowBox = new Gtk.HBox ();
			WindowBox.Spacing = 24;
			WindowBox.BorderWidth = 8;
			view.Add (WindowBox);
			
			Gtk.Image icon = new Gtk.Image ();
			icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Reminder-48.png"));
			icon.Yalign = (float) 0.00;
			WindowBox.Add (icon);
			
			Gtk.VBox LabelsBox = new Gtk.VBox ();
			LabelsBox.Spacing  = 12;
			WindowBox.Add (LabelsBox);
			
			Gtk.HBox TitleBox = new Gtk.HBox ();
			TitleBox.Spacing  = 8;
			LabelsBox.Add (TitleBox);
			
			Gtk.Image PriorityImage = new Gtk.Image (priority);
			TitleBox.PackStart (PriorityImage, false, true, 0);
			
			Gtk.Label TitleLabel = new Gtk.Label ();
			TitleLabel.UseMarkup = true;
			TitleLabel.SetAlignment ((float) 0.00, (float) 0.50);
			TitleLabel.Markup    = "<span weight=\"bold\" size=\"larger\">" + Catalog.GetString ("Reminder") + "</span>";
			TitleBox.PackStart (TitleLabel, true, true, 0);
			
			Gtk.Label MessageLabel    = new Gtk.Label ();
			MessageLabel.UseMarkup    = true;
			MessageLabel.Markup       = message;
			MessageLabel.Xalign       = (float) 0.00;
			MessageLabel.Wrap         = true;
			MessageLabel.WidthRequest = 300;
			LabelsBox.Add (MessageLabel);
			
			Gtk.HButtonBox ButtonsBox = new Gtk.HButtonBox ();
			ButtonsBox.Layout         = Gtk.ButtonBoxStyle.End;
			ButtonsBox.Spacing        = 6;
			LabelsBox.PackStart (ButtonsBox, true, true, 0);
			
			Gtk.Button btn1 = new Gtk.Button ();
			btn1.Relief     = Gtk.ReliefStyle.None;
			btn1.Clicked += new EventHandler (on_Btn1_clicked);
			Gtk.HBox Btn1HBox = new Gtk.HBox ();
			btn1.Add (Btn1HBox);
			Gtk.Label Btn1Label = new Gtk.Label ("");
			Btn1Label.UseMarkup = true;
			Btn1Label.Markup    = "<span color=\"#1111EE\">" + Catalog.GetString ("Completed !") + "</span>";
			Btn1HBox.Add (Btn1Label);
			ButtonsBox.Add (btn1);
			
			Gtk.Button btn2 = new Gtk.Button ();
			btn2.Relief     = Gtk.ReliefStyle.None;
			btn2.Clicked += new EventHandler (on_Btn2_clicked);
			Gtk.HBox Btn2HBox = new Gtk.HBox ();
			btn2.Add (Btn2HBox);
			Gtk.Label Btn2Label = new Gtk.Label ("");
			Btn2Label.UseMarkup = true;
			Btn2Label.Markup    = "<span color=\"#1111EE\">" + Catalog.GetString ("I'll do it later...") + "</span>";
			Btn2HBox.Add (Btn2Label);
			ButtonsBox.Add (btn2);
			
			this.TasksListStore = TasksListStore;
			this.TaskIter       = TaskIter;
			
			WindowBox.ShowAll ();
		}
		
		
		public TrayMessage (string message, int priority, int TaskID, DateTime date)
		{
			this.reminder = true;
			this.TaskID = TaskID;
			this.EntryDate = date;
			
			window = new Gtk.Window (Gtk.WindowType.Toplevel);
			window.TypeHint = Gdk.WindowTypeHint.Normal;
			window.Stick ();
			window.KeepAbove = true;
			window.SkipTaskbarHint = true;
			window.Decorated = false;
			
			Gtk.Viewport view = new Gtk.Viewport ();
			view.ModifyBg (Gtk.StateType.Normal , new Gdk.Color (255, 255, 255));
			window.Add (view);
			
			Gtk.HBox WindowBox = new Gtk.HBox ();
			WindowBox.Spacing = 24;
			WindowBox.BorderWidth = 8;
			view.Add (WindowBox);
			
			Gtk.Image icon = new Gtk.Image ();
			icon = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Reminder-48.png"));
			icon.Yalign = (float) 0.00;
			WindowBox.Add (icon);
			
			Gtk.VBox LabelsBox = new Gtk.VBox ();
			LabelsBox.Spacing  = 12;
			WindowBox.Add (LabelsBox);
			
			Gtk.HBox TitleBox = new Gtk.HBox ();
			TitleBox.Spacing  = 8;
			LabelsBox.Add (TitleBox);
			
			Gtk.Image PriorityImage = new Gtk.Image (new Gdk.Pixbuf (Assembly.GetEntryAssembly (), "Priority-" + priority + ".png"));
			TitleBox.PackStart (PriorityImage, false, true, 0);
			
			Gtk.Label TitleLabel = new Gtk.Label ();
			TitleLabel.UseMarkup = true;
			TitleLabel.SetAlignment ((float) 0.00, (float) 0.50);
			TitleLabel.Markup    = "<span weight=\"bold\" size=\"larger\">" + Catalog.GetString ("Reminder") + "</span>";
			TitleBox.PackStart (TitleLabel, true, true, 0);
			
			Gtk.Label MessageLabel    = new Gtk.Label ();
			MessageLabel.UseMarkup    = true;
			MessageLabel.Markup       = message;
			MessageLabel.Xalign       = (float) 0.00;
			MessageLabel.Wrap         = true;
			MessageLabel.WidthRequest = 300;
			LabelsBox.Add (MessageLabel);
			
			Gtk.HButtonBox ButtonsBox = new Gtk.HButtonBox ();
			ButtonsBox.Layout         = Gtk.ButtonBoxStyle.End;
			ButtonsBox.Spacing        = 6;
			LabelsBox.PackStart (ButtonsBox, true, true, 0);
			
			Gtk.Button btn1 = new Gtk.Button ();
			btn1.Relief     = Gtk.ReliefStyle.None;
			btn1.Clicked += new EventHandler (on_Btn1_clicked);
			Gtk.HBox Btn1HBox = new Gtk.HBox ();
			btn1.Add (Btn1HBox);
			Gtk.Label Btn1Label = new Gtk.Label ("");
			Btn1Label.UseMarkup = true;
			Btn1Label.Markup    = "<span color=\"#1111EE\">" + Catalog.GetString ("Completed !") + "</span>";
			Btn1HBox.Add (Btn1Label);
			ButtonsBox.Add (btn1);
			
			Gtk.Button btn2 = new Gtk.Button ();
			btn2.Relief     = Gtk.ReliefStyle.None;
			btn2.Clicked += new EventHandler (on_Btn2_clicked);
			Gtk.HBox Btn2HBox = new Gtk.HBox ();
			btn2.Add (Btn2HBox);
			Gtk.Label Btn2Label = new Gtk.Label ("");
			Btn2Label.UseMarkup = true;
			Btn2Label.Markup    = "<span color=\"#1111EE\">" + Catalog.GetString ("I'll do it later...") + "</span>";
			Btn2HBox.Add (Btn2Label);
			ButtonsBox.Add (btn2);
			
			this.TasksListStore = TasksListStore;
			this.TaskIter       = TaskIter;
			
			WindowBox.ShowAll ();
		}
		

		private void on_ExitBtn_clicked (object o, EventArgs args)
		{
			this.Hide ();
		}
		
		
		private void on_Btn1_clicked (object o, EventArgs args)
		{
			if (EntryDate == DateTime.Today)
				TasksListStore.SetValue (TaskIter, 0, true);
			else {
				gnutu.Entry ent = gnutu.DB.GetEntryByDate (DateTime.Today);
				gnutu.ToDo task = (gnutu.ToDo) ent.todo [TaskID];
				task.did = true;
				ent.todo [TaskID] = task;
			}
			
			this.Hide ();
		}
		
		
		private void on_Btn2_clicked (object o, EventArgs args)
		{
			this.Hide ();
		}
		
		
		public void Show ()
		{
			window.Move (30, 30);
			window.ShowAll ();
			
			/*if (this.reminder == true) {
				// flying window :-)
				int a,b, x;
				window.GetSize (out a, out b);
				
				x = window.Screen.Width  - a;
				window.Sensitive = false;
				
				while (x > 10) {
					if (x > 10)
						x--;
					
					System.Threading.Thread.Sleep (1);
					
					window.Move (x, 30);
					while (Gtk.Application.EventsPending ())
		       			Gtk.Application.RunIteration ();
				}
			} else
				window.Move (30, 30);*/
			
			window.Sensitive = true;
		}
		
		
		public void Hide ()
		{
			window.Destroy ();
		}
	}
}
